#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/2/28 17:00
Desc: 中国金融期货交易所-数据-交易参数
http://www.cffex.com.cn/jycs/
"""

import xml.etree.ElementTree as ET

import pandas as pd
import requests


def futures_contract_info_cffex(date: str = "20240228") -> pd.DataFrame:
    """
    中国金融期货交易所-数据-交易参数
    http://www.gfex.com.cn/gfex/hyxx/ywcs.shtml
    :param date: 查询日期
    :type date: str
    :return: 交易参数汇总查询
    :rtype: pandas.DataFrame
    """
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
    }
    url = f"http://www.cffex.com.cn/sj/jycs/{date[:6]}/{date[6:]}/index.xml"
    r = requests.get(url, headers=headers)
    xml_data = r.text
    # 解析 XML
    tree = ET.ElementTree(ET.fromstring(xml_data))
    root = tree.getroot()
    # 获取所有的记录
    records = root.findall(".//INDEX")
    # 解析数据并填充到列表中
    data = []
    for record in records:
        # 对于每个记录，创建一个字典
        row_data = {}
        for field in record:
            row_data[field.tag] = field.text
        # 将字典添加到数据列表中
        data.append(row_data)

    temp_df = pd.DataFrame(data)
    temp_df.rename(
        columns={
            "TRADING_DAY": "查询交易日",
            "PRODUCT_ID": "品种",
            "INSTRUMENT_ID": "合约代码",
            "INSTRUMENT_MONTH": "合约月份",
            "BASIS_PRICE": "挂盘基准价",
            "OPEN_DATE": "上市日",
            "END_TRADING_DAY": "最后交易日",
            "UPPER_VALUE": "涨停板幅度",
            "LOWER_VALUE": "跌停板幅度",
            "UPPERLIMITPRICE": "涨停板价位",
            "LOWERLIMITPRICE": "跌停板价位",
            "LONG_LIMIT": "持仓限额",
        },
        inplace=True,
    )
    temp_df["挂盘基准价"] = pd.to_numeric(temp_df["挂盘基准价"], errors="coerce")
    temp_df["涨停板价位"] = pd.to_numeric(temp_df["涨停板价位"], errors="coerce")
    temp_df["跌停板价位"] = pd.to_numeric(temp_df["跌停板价位"], errors="coerce")
    temp_df["持仓限额"] = pd.to_numeric(temp_df["持仓限额"], errors="coerce")

    temp_df["查询交易日"] = pd.to_datetime(
        temp_df["查询交易日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["上市日"] = pd.to_datetime(
        temp_df["上市日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["最后交易日"] = pd.to_datetime(
        temp_df["最后交易日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["查询交易日"] = pd.to_datetime(
        temp_df["查询交易日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df = temp_df[
        [
            "合约代码",
            "合约月份",
            "挂盘基准价",
            "上市日",
            "最后交易日",
            "涨停板幅度",
            "跌停板幅度",
            "涨停板价位",
            "跌停板价位",
            "持仓限额",
            "品种",
            "查询交易日",
        ]
    ]
    temp_df.sort_values(["合约代码"], ascending=False, ignore_index=True, inplace=True)
    return temp_df


if __name__ == "__main__":
    futures_contract_info_cffex_df = futures_contract_info_cffex(date="20240228")
    print(futures_contract_info_cffex_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/5 18:00
Desc: 外盘期货-历史行情数据-日频率
https://finance.sina.com.cn/money/future/hf.html
"""

from datetime import datetime
from io import StringIO

import pandas as pd
import requests

from akshare.futures.futures_hq_sina import (
    futures_foreign_commodity_subscribe_exchange_symbol,
)


def futures_foreign_hist(symbol: str = "ZSD") -> pd.DataFrame:
    """
    外盘期货-历史行情数据-日频率
    https://finance.sina.com.cn/money/future/hf.html
    :param symbol: 外盘期货代码, 可以通过 ak.futures_foreign_commodity_subscribe_exchange_symbol() 来获取所有品种代码
    :type symbol: str
    :return: 历史行情数据-日频率
    :rtype: pandas.DataFrame
    """
    today = f"{datetime.today().year}_{datetime.today().month}_{datetime.today().day}"
    url = (
        f"https://stock2.finance.sina.com.cn/futures/api/jsonp.php/var%20_S{today}=/"
        f"GlobalFuturesService.getGlobalFuturesDailyKLine"
    )
    params = {
        "symbol": symbol,
        "_": today,
        "source": "web",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_df = pd.read_json(StringIO(data_text[data_text.find("[") : -2]))
    return data_df


def futures_foreign_detail(symbol: str = "ZSD") -> pd.DataFrame:
    """
    foreign futures contract detail data
    :param symbol: futures symbol, you can get it from hf_subscribe_exchange_symbol function
    :type symbol: str
    :return: contract detail
    :rtype: pandas.DataFrame
    """
    url = f"https://finance.sina.com.cn/futures/quotes/{symbol}.shtml"
    r = requests.get(url)
    r.encoding = "gbk"
    data_text = r.text
    data_df = pd.read_html(StringIO(data_text))[6]
    return data_df


if __name__ == "__main__":
    futures_foreign_hist_df = futures_foreign_hist(symbol="JY")
    print(futures_foreign_hist_df)

    subscribes = futures_foreign_commodity_subscribe_exchange_symbol()

    for item in subscribes:
        futures_foreign_detail_df = futures_foreign_detail(symbol=item)
        print(futures_foreign_detail_df)

    for item in subscribes:
        futures_foreign_hist_df = futures_foreign_hist(symbol=item)
        print(futures_foreign_hist_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/11/30 18:00
Desc: 期货日线行情
"""

import datetime
import json
import re
import zipfile
from io import BytesIO, StringIO

import numpy as np
import pandas as pd
import requests

from akshare.futures import cons
from akshare.futures.requests_fun import requests_link

calendar = cons.get_calendar()


def _futures_daily_czce(
    date: str = "20100824", dataset: str = "datahistory2010"
) -> pd.DataFrame:
    """
    郑州商品交易所-交易数据-历史行情下载
    http://www.czce.com.cn/cn/jysj/lshqxz/H770319index_1.htm
    :param date: 需要的日期
    :type date: str
    :param dataset: 数据集的名称; 此处只需要替换 datahistory2010 中的 2010 即可
    :type dataset: str
    :return: 指定日期的所有品种行情数据
    :rtype: pandas.DataFrame
    """
    url = f"http://www.czce.com.cn/cn/exchange/{dataset}.zip"
    r = requests.get(url)
    with zipfile.ZipFile(BytesIO(r.content)) as file:
        with file.open(f"{dataset}.txt") as my_file:
            data = my_file.read().decode("gb2312")
            data_df = pd.read_table(StringIO(data), sep=r"|", header=1)
            data_df.columns = [item.strip() for item in data_df.columns]
            data_df.dropna(axis=1, inplace=True)
            for column in data_df.columns:
                try:
                    data_df[column] = data_df[column].str.strip("\t")
                    data_df[column] = data_df[column].str.replace(",", "")
                except:  # noqa: E722
                    data_df[column] = data_df[column]
    data_df["昨结算"] = pd.to_numeric(data_df["昨结算"])
    data_df["今开盘"] = pd.to_numeric(data_df["今开盘"])
    data_df["最高价"] = pd.to_numeric(data_df["最高价"])
    data_df["最低价"] = pd.to_numeric(data_df["最低价"])
    data_df["今收盘"] = pd.to_numeric(data_df["今收盘"])
    data_df["今结算"] = pd.to_numeric(data_df["今结算"])
    data_df["涨跌1"] = pd.to_numeric(data_df["涨跌1"])
    data_df["涨跌2"] = pd.to_numeric(data_df["涨跌2"])
    data_df["成交量(手)"] = pd.to_numeric(data_df["成交量(手)"])
    data_df["空盘量"] = pd.to_numeric(data_df["空盘量"])
    data_df["增减量"] = pd.to_numeric(data_df["增减量"])
    data_df["成交额(万元)"] = pd.to_numeric(data_df["成交额(万元)"])
    data_df["交割结算价"] = pd.to_numeric(data_df["交割结算价"])
    data_df["交易日期"] = pd.to_datetime(data_df["交易日期"])
    data_df.columns = [
        "date",
        "symbol",
        "pre_settle",
        "open",
        "high",
        "low",
        "close",
        "settle",
        "-",
        "-",
        "volume",
        "open_interest",
        "-",
        "turnover",
        "-",
    ]
    variety_list = [
        re.compile(r"[a-zA-Z_]+").findall(item)[0] for item in data_df["symbol"]
    ]
    data_df["variety"] = variety_list
    data_df = data_df[
        [
            "symbol",
            "date",
            "open",
            "high",
            "low",
            "close",
            "volume",
            "open_interest",
            "turnover",
            "settle",
            "pre_settle",
            "variety",
        ]
    ]
    temp_df = data_df[data_df["date"] == pd.Timestamp(date)].copy()
    temp_df["date"] = date
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


def get_cffex_daily(date: str = "20100416") -> pd.DataFrame:
    """
    中国金融期货交易所-日频率交易数据
    http://www.cffex.com.cn/rtj/
    :param date: 交易日; 数据开始时间为 20100416
    :type date: str
    :return: 日频率交易数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return pd.DataFrame()
    url = f"http://www.cffex.com.cn/sj/historysj/{date[:-2]}/zip/{date[:-2]}.zip"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/108.0.0.0 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    try:
        with zipfile.ZipFile(BytesIO(r.content)) as file:
            with file.open(f"{date}_1.csv") as my_file:
                data = my_file.read().decode("gb2312")
                data_df = pd.read_csv(StringIO(data))
    except:  # noqa: E722
        return pd.DataFrame()
    data_df = data_df[data_df["合约代码"] != "小计"]
    data_df = data_df[data_df["合约代码"] != "合计"]
    data_df = data_df[~data_df["合约代码"].str.contains("IO")]
    data_df = data_df[~data_df["合约代码"].str.contains("MO")]
    data_df = data_df[~data_df["合约代码"].str.contains("HO")]
    data_df.reset_index(inplace=True, drop=True)
    data_df["合约代码"] = data_df["合约代码"].str.strip()
    symbol_list = data_df["合约代码"].to_list()
    variety_list = [re.compile(r"[a-zA-Z_]+").findall(item)[0] for item in symbol_list]
    if data_df.shape[1] == 15:
        data_df.columns = [
            "symbol",
            "open",
            "high",
            "low",
            "volume",
            "turnover",
            "open_interest",
            "_",
            "close",
            "settle",
            "pre_settle",
            "_",
            "_",
            "_",
            "_",
        ]
    else:
        data_df.columns = [
            "symbol",
            "open",
            "high",
            "low",
            "volume",
            "turnover",
            "open_interest",
            "_",
            "close",
            "settle",
            "pre_settle",
            "_",
            "_",
            "_",
        ]
    data_df["date"] = date
    data_df["variety"] = variety_list
    data_df = data_df[
        [
            "symbol",
            "date",
            "open",
            "high",
            "low",
            "close",
            "volume",
            "open_interest",
            "turnover",
            "settle",
            "pre_settle",
            "variety",
        ]
    ]
    return data_df


def get_gfex_daily(date: str = "20221223") -> pd.DataFrame:
    """
    广州期货交易所-日频率-量价数据
    广州期货交易所: 工业硅(上市时间: 20221222)
    http://www.gfex.com.cn/gfex/rihq/hqsj_tjsj.shtml
    :param date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日
    :type date: str or datetime.date
    :return: 广州期货交易所-日频率-量价数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn(f"{day.strftime('%Y%m%d')}非交易日")
        return pd.DataFrame()
    url = "http://www.gfex.com.cn/u/interfacesWebTiDayQuotes/loadList"
    payload = {"trade_date": date, "trade_type": "0"}
    headers = {
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "32",
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Host": "www.gfex.com.cn",
        "Origin": "http://www.gfex.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://www.gfex.com.cn/gfex/rihq/hqsj_tjsj.shtml",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/108.0.0.0 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
        "content-type": "application/x-www-form-urlencoded",
    }
    r = requests.post(url, data=payload, headers=headers)
    try:
        data_json = r.json()
    except:  # noqa: E722
        return pd.DataFrame()
    result_df = pd.DataFrame(data_json["data"])
    result_df = result_df[~result_df["variety"].str.contains("小计")]
    result_df = result_df[~result_df["variety"].str.contains("总计")]
    result_df["symbol"] = (
        result_df["varietyOrder"].str.upper() + result_df["delivMonth"]
    )
    result_df["date"] = date
    result_df["open"] = pd.to_numeric(result_df["open"], errors="coerce")
    result_df["high"] = pd.to_numeric(result_df["high"], errors="coerce")
    result_df["low"] = pd.to_numeric(result_df["low"], errors="coerce")
    result_df["close"] = pd.to_numeric(result_df["close"], errors="coerce")
    result_df["volume"] = pd.to_numeric(result_df["volumn"], errors="coerce")
    result_df["open_interest"] = pd.to_numeric(
        result_df["openInterest"], errors="coerce"
    )
    result_df["turnover"] = pd.to_numeric(result_df["turnover"], errors="coerce")
    result_df["settle"] = pd.to_numeric(result_df["clearPrice"], errors="coerce")
    result_df["pre_settle"] = pd.to_numeric(result_df["lastClear"], errors="coerce")
    result_df["variety"] = result_df["varietyOrder"].str.upper()
    result_df = result_df[
        [
            "symbol",
            "date",
            "open",
            "high",
            "low",
            "close",
            "volume",
            "open_interest",
            "turnover",
            "settle",
            "pre_settle",
            "variety",
        ]
    ]
    return result_df


def get_ine_daily(date: str = "20241129") -> pd.DataFrame:
    """
    上海国际能源交易中心-日频率-量价数据
    上海国际能源交易中心: 原油期货(上市时间: 20180326); 20号胶期货(上市时间: 20190812)
    trade_price: https://www.ine.cn/statements/daily/?paramid=kx
    trade_note: https://www.ine.cn/data/datanote.dat
    :param date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日
    :type date: str or datetime.date
    :return: 上海国际能源交易中心-日频率-量价数据
    :rtype: pandas.DataFrame or None
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn(f"{day.strftime('%Y%m%d')}非交易日")
        return pd.DataFrame()
    url = f"https://www.ine.cn/data/tradedata/future/dailydata/kx{day.strftime('%Y%m%d')}.dat"
    r = requests.get(url, headers=cons.shfe_headers)
    result_df = pd.DataFrame()
    try:
        data_json = r.json()
    except:  # noqa: E722
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["o_curinstrument"]).iloc[:-1, :]
    temp_df = temp_df[temp_df["DELIVERYMONTH"] != "小计"]
    temp_df = temp_df[~temp_df["PRODUCTNAME"].str.contains("总计")]
    try:
        result_df["symbol"] = (
            temp_df["PRODUCTGROUPID"].str.upper().str.strip() + temp_df["DELIVERYMONTH"]
        )
    except:  # noqa: E722
        result_df["symbol"] = (
            temp_df["PRODUCTID"]
            .str.upper()
            .str.strip()
            .str.split("_", expand=True)
            .iloc[:, 0]
            + temp_df["DELIVERYMONTH"]
        )
    result_df["date"] = day.strftime("%Y%m%d")
    result_df["open"] = temp_df["OPENPRICE"]
    result_df["high"] = temp_df["HIGHESTPRICE"]
    result_df["low"] = temp_df["LOWESTPRICE"]
    result_df["close"] = temp_df["CLOSEPRICE"]
    result_df["volume"] = temp_df["VOLUME"]
    result_df["open_interest"] = temp_df["OPENINTEREST"]
    try:
        result_df["turnover"] = temp_df["TURNOVER"]
    except:  # noqa: E722
        result_df["turnover"] = 0
    result_df["settle"] = temp_df["SETTLEMENTPRICE"]
    result_df["pre_settle"] = temp_df["PRESETTLEMENTPRICE"]
    try:
        result_df["variety"] = temp_df["PRODUCTGROUPID"].str.upper().str.strip()
    except:  # noqa: E722
        result_df["variety"] = (
            temp_df["PRODUCTID"]
            .str.upper()
            .str.strip()
            .str.split("_", expand=True)
            .iloc[:, 0]
        )
    result_df = result_df[result_df["symbol"] != "总计"]
    result_df = result_df[~result_df["symbol"].str.contains("efp")]
    return result_df


def get_czce_daily(date: str = "20050525") -> pd.DataFrame:
    """
    郑州商品交易所-日频率-量价数据
    http://www.czce.com.cn/cn/jysj/mrhq/H770301index_1.htm
    :param date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date 对象，默认为当前交易日; 日期需要大于 20100824
    :type date: str or datetime.date
    :return: 郑州商品交易所-日频率-量价数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/108.0.0.0 Safari/537.36"
    }
    url = ""
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn(f"{day.strftime('%Y%m%d')}非交易日")
        return pd.DataFrame()
    if day > datetime.date(2010, 8, 24):
        if day > datetime.date(2015, 11, 11):
            u = cons.CZCE_DAILY_URL_3
            url = u % (day.strftime("%Y"), day.strftime("%Y%m%d"))
        elif day <= datetime.date(2015, 11, 11):
            u = cons.CZCE_DAILY_URL_2
            url = u % (day.strftime("%Y"), day.strftime("%Y%m%d"))
        listed_columns = cons.CZCE_COLUMNS
        output_columns = cons.OUTPUT_COLUMNS
        try:
            r = requests.get(url, headers=headers)
            if datetime.date(2015, 11, 12) <= day <= datetime.date(2017, 12, 27):
                html = str(r.content, encoding="gbk")
            else:
                html = r.text
        except requests.exceptions.HTTPError as reason:
            if reason.response.status_code != 404:
                print(
                    cons.CZCE_DAILY_URL_3
                    % (day.strftime("%Y"), day.strftime("%Y%m%d")),
                    reason,
                )
            return pd.DataFrame()
        if html.find("您的访问出错了") >= 0 or html.find("无期权每日行情交易记录") >= 0:
            return pd.DataFrame()
        html = [
            i.replace(" ", "").split("|")
            for i in html.split("\n")[:-3]
            if i[0][0] != "小"
        ]

        if day > datetime.date(2015, 11, 11):
            if html[1][0] not in ["品种月份", "品种代码", "合约代码"]:
                return pd.DataFrame()
            dict_data = list()
            day_const = int(day.strftime("%Y%m%d"))
            for row in html[2:]:
                m = cons.FUTURES_SYMBOL_PATTERN.match(row[0])
                if not m:
                    continue
                row_dict = {
                    "date": day_const,
                    "symbol": row[0],
                    "variety": m.group(1),
                }
                for i, field in enumerate(listed_columns):
                    if row[i + 1] == "\r" or row[i + 1] == "":
                        row_dict[field] = 0.0
                    elif field in [
                        "volume",
                        "open_interest",
                        "oi_chg",
                        "exercise_volume",
                    ]:
                        row[i + 1] = row[i + 1].replace(",", "")
                        row_dict[field] = int(row[i + 1])
                    else:
                        row[i + 1] = row[i + 1].replace(",", "")
                        row_dict[field] = float(row[i + 1])
                dict_data.append(row_dict)
            return pd.DataFrame(dict_data)[output_columns]
        elif day <= datetime.date(2015, 11, 11):
            dict_data = list()
            day_const = int(day.strftime("%Y%m%d"))
            for row in html[1:]:
                row = row[0].split(",")
                m = cons.FUTURES_SYMBOL_PATTERN.match(row[0])
                if not m:
                    continue
                row_dict = {
                    "date": day_const,
                    "symbol": row[0],
                    "variety": m.group(1),
                }
                for i, field in enumerate(listed_columns):
                    if row[i + 1] == "\r":
                        row_dict[field] = 0.0
                    elif field in [
                        "volume",
                        "open_interest",
                        "oi_chg",
                        "exercise_volume",
                    ]:
                        row_dict[field] = int(float(row[i + 1]))
                    else:
                        row_dict[field] = float(row[i + 1])
                dict_data.append(row_dict)
            return pd.DataFrame(dict_data)[output_columns]

    if day <= datetime.date(2010, 8, 24):
        _futures_daily_czce_df = _futures_daily_czce(date)
        return _futures_daily_czce_df


def get_shfe_daily(date: str = "20220415") -> pd.DataFrame:
    """
    上海期货交易所-日频率-量价数据
    https://tsite.shfe.com.cn/statements/dataview.html?paramid=kx
    :param date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象, 默认为当前交易日
    :type date: str or datetime.date
    :return: 上海期货交易所-日频率-量价数据
    :rtype: pandas.DataFrame or None
    上期所日交易数据(DataFrame):
    symbol        合约代码
    date          日期
    open          开盘价
    high          最高价
    low           最低价
    close         收盘价
    volume        成交量
    open_interest 持仓量
    turnover      成交额
    settle        结算价
    pre_settle     前结算价
    variety       合约类别
    或 None(给定交易日没有交易数据)
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return pd.DataFrame()
    try:
        json_data = json.loads(
            requests_link(
                cons.SHFE_DAILY_URL_20250630 % (day.strftime("%Y%m%d")),
                headers=cons.shfe_headers,
            ).text
        )
    except requests.HTTPError as reason:
        if reason.response != 404:
            print(cons.SHFE_DAILY_URL_20250630 % (day.strftime("%Y%m%d")), reason)
        return pd.DataFrame()

    if len(json_data["o_curinstrument"]) == 0:
        return pd.DataFrame()

    df = pd.DataFrame(
        [
            row
            for row in json_data["o_curinstrument"]
            if row["DELIVERYMONTH"] not in ["小计", "合计"]
            and row["DELIVERYMONTH"] != ""
        ]
    )
    try:
        df["variety"] = df["PRODUCTGROUPID"].str.upper().str.strip()
    except KeyError:
        df["variety"] = (
            df["PRODUCTID"]
            .str.upper()
            .str.split("_", expand=True)
            .iloc[:, 0]
            .str.strip()
        )
    df["symbol"] = df["variety"] + df["DELIVERYMONTH"]
    df["date"] = day.strftime("%Y%m%d")
    df["VOLUME"] = df["VOLUME"].apply(lambda x: 0 if x == "" else x)
    try:
        df["turnover"] = df["TURNOVER"].apply(lambda x: 0 if x == "" else x)
    except KeyError:
        df["turnover"] = np.nan
    df.rename(columns=cons.SHFE_COLUMNS, inplace=True)
    df = df[~df["symbol"].str.contains("efp")]
    df = df[cons.OUTPUT_COLUMNS]
    df.reset_index(inplace=True)
    return df


def get_dce_daily(date: str = "20251027") -> pd.DataFrame:
    """
    大连商品交易所日交易数据
    http://www.dce.com.cn/dalianshangpin/xqsj/tjsj26/rtj/rxq/index.html
    :param date: 交易日, e.g., 20200416
    :type date: str
    :return: 具体交易日的个品种行情数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return pd.DataFrame()
    url = "http://www.dce.com.cn/dcereport/publicweb/dailystat/dayQuotes"
    payload = {
        "contractId": "",
        "lang": "zh",
        "optionSeries": "",
        "statisticsType": "0",
        "tradeDate": date,
        "tradeType": "1",
        "varietyId": "all",
    }
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.rename(
        columns={
            "variety": "品种名称",
            "contractId": "合约",
            "open": "开盘价",
            "high": "最高价",
            "low": "最低价",
            "close": "收盘价",
            "lastClear": "前结算价",
            "clearPrice": "结算价",
            "diff": "涨跌",
            "diff1": "涨跌1",
            "volumn": "成交量",
            "openInterest": "持仓量",
            "diffI": "持仓量变化",
            "turnover": "成交额",
        },
        inplace=True,
    )
    temp_df = temp_df[~temp_df["品种名称"].str.contains("小计")]
    temp_df = temp_df[~temp_df["品种名称"].str.contains("总计")]
    temp_df["variety"] = temp_df["品种名称"].map(lambda x: cons.DCE_MAP[x])
    temp_df["symbol"] = temp_df["合约"]
    del temp_df["品种名称"]
    del temp_df["合约"]
    temp_df.columns = [
        "open",
        "high",
        "low",
        "close",
        "pre_settle",
        "settle",
        "_",
        "_",
        "_",
        "volume",
        "open_interest",
        "_",
        "turnover",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "variety",
        "symbol",
    ]
    temp_df["date"] = date
    temp_df = temp_df[
        [
            "symbol",
            "date",
            "open",
            "high",
            "low",
            "close",
            "volume",
            "open_interest",
            "turnover",
            "settle",
            "pre_settle",
            "variety",
        ]
    ]
    temp_df = temp_df.astype(
        {
            "open": "float",
            "high": "float",
            "low": "float",
            "close": "float",
            "volume": "float",
            "open_interest": "float",
            "turnover": "float",
            "settle": "float",
            "pre_settle": "float",
        }
    )
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


def get_futures_daily(
    start_date: str = "20220208",
    end_date: str = "20220208",
    market: str = "CFFEX",
) -> pd.DataFrame:
    """
    交易所日交易数据
    :param start_date: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
    :type start_date: str
    :param end_date: 结束数据 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
    :type end_date: str
    :param market: 'CFFEX' 中金所, 'CZCE' 郑商所,  'SHFE' 上期所, 'DCE' 大商所 之一, 'INE' 上海国际能源交易中心, "GFEX" 广州期货交易所。默认为中金所
    :type market: str
    :return: 交易所日交易数据
    :rtype: pandas.DataFrame
    """
    if market.upper() == "CFFEX":
        f = get_cffex_daily
    elif market.upper() == "CZCE":
        f = get_czce_daily
    elif market.upper() == "SHFE":
        f = get_shfe_daily
    elif market.upper() == "DCE":
        f = get_dce_daily
    elif market.upper() == "INE":
        f = get_ine_daily
    elif market.upper() == "GFEX":
        f = get_gfex_daily
    else:
        print("Invalid Market Symbol")
        return pd.DataFrame()

    start_date = (
        cons.convert_date(start_date)
        if start_date is not None
        else datetime.date.today()
    )
    end_date = (
        cons.convert_date(end_date)
        if end_date is not None
        else cons.convert_date(cons.get_latest_data_date(datetime.datetime.now()))
    )

    df_list = list()
    while start_date <= end_date:
        df = f(date=str(start_date).replace("-", ""))
        if not df.empty:
            df_list.append(df)
        start_date += datetime.timedelta(days=1)

    if len(df_list) == 0:
        return pd.DataFrame()
    elif len(df_list) > 0:
        temp_df = pd.concat(df_list).reset_index(drop=True)
        temp_df = temp_df[~temp_df["symbol"].str.contains("efp")]
        return temp_df
    else:
        return pd.DataFrame()


if __name__ == "__main__":
    get_futures_daily_df = get_futures_daily(
        start_date="20250708", end_date="20250708", market="DCE"
    )
    print(get_futures_daily_df)

    get_dce_daily_df = get_dce_daily(date="20251029")
    print(get_dce_daily_df)

    get_cffex_daily_df = get_cffex_daily(date="20230810")
    print(get_cffex_daily_df)

    get_ine_daily_df = get_ine_daily(date="20230818")
    print(get_ine_daily_df)

    get_czce_daily_df = get_czce_daily(date="20210513")
    print(get_czce_daily_df)

    get_shfe_daily_df = get_shfe_daily(date="20250630")
    print(get_shfe_daily_df)

    get_gfex_daily_df = get_gfex_daily(date="20221228")
    print(get_gfex_daily_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/9/24 15:00
Desc: 东方财富-港股-行业对比
https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/IndustryComparison
"""

import pandas as pd
import requests


def stock_hk_growth_comparison_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-行业对比-成长性对比
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/IndustryComparison
    :param symbol: 股票代码
    :type symbol: str
    :return: 成长性对比
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "reportName": "RPT_PCF10_INDUSTRY_HKGROWTH",
        "columns": "SECUCODE,SECURITY_CODE,ORG_CODE,REPORT_DATE,TYPE_ID,TYPE_TYPE,"
        "TYPE_NAME,TYPE_NAME_EN,CORRE_SECURITY_CODE,CORRE_SECUCODE,"
        "CORRE_SECURITY_NAME,EPS_YOY,OPERATE_INCOME_YOY,OPERATE_PROFIT_YOY,"
        "TOTAL_ASSET_YOY,EPS_YOY_RANK,OPINCOME_YOY_RANK,OPROFIT_YOY_RANK,TOASSET_YOY_RANK",
        "quoteColumns": "",
        "filter": f'(SECUCODE="{symbol}.HK")(CORRE_SECUCODE="{symbol}.HK")',
        "pageNumber": "1",
        "pageSize": "",
        "sortTypes": "",
        "sortColumns": "",
        "source": "F10",
        "client": "PC",
        "v": "03313416193688571",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    field_mapping = {
        "CORRE_SECURITY_CODE": "代码",
        "CORRE_SECURITY_NAME": "简称",
        "EPS_YOY": "基本每股收益同比增长率",
        "EPS_YOY_RANK": "基本每股收益同比增长率排名",
        "OPERATE_INCOME_YOY": "营业收入同比增长率",
        "OPINCOME_YOY_RANK": "营业收入同比增长率排名",
        "OPERATE_PROFIT_YOY": "营业利润率同比增长率",
        "OPROFIT_YOY_RANK": "营业利润率同比增长率排名",
        "TOTAL_ASSET_YOY": "基本每股收总资产同比增长率益同比增长率",
        "TOASSET_YOY_RANK": "总资产同比增长率排名",
    }
    temp_df = pd.DataFrame(columns=field_mapping.values())
    if data_json["result"] is not None:
        temp_df = pd.DataFrame(data_json["result"]["data"])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[field_mapping.values()]
    return temp_df


def stock_hk_valuation_comparison_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-行业对比-估值对比
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/IndustryComparison
    :param symbol: 股票代码
    :type symbol: str
    :return: 估值对比
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "reportName": "RPT_PCF10_INDUSTRY_HKCVALUE",
        "columns": "SECUCODE,SECURITY_CODE,ORG_CODE,REPORT_DATE,TYPE_ID,"
        "TYPE_TYPE,TYPE_NAME,TYPE_NAME_EN,CORRE_SECURITY_CODE,"
        "CORRE_SECUCODE,CORRE_SECURITY_NAME,PE_TTM,PE_LYR,PB_MQR,"
        "PB_LYR,PS_TTM,PS_LYR,PCE_TTM,PCE_LYR,PE_TTM_RANK,PE_LYR_RANK,"
        "PB_MQR_RANK,PB_LYR_RANK,PS_TTM_RANK,PS_LYR_RANK,PCE_TTM_RANK,PCE_LYR_RANK",
        "quoteColumns": "",
        "filter": f'(SECUCODE="{symbol}.HK")(CORRE_SECUCODE="{symbol}.HK")',
        "pageNumber": "1",
        "pageSize": "",
        "sortTypes": "",
        "sortColumns": "",
        "source": "F10",
        "client": "PC",
        "v": "03445297742754925",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    field_mapping = {
        "CORRE_SECURITY_CODE": "代码",
        "CORRE_SECURITY_NAME": "简称",
        "PE_TTM": "市盈率-TTM",
        "PE_TTM_RANK": "市盈率-TTM排名",
        "PE_LYR": "市盈率-LYR",
        "PE_LYR_RANK": "市盈率-LYR排名",
        "PB_MQR": "市净率-MRQ",
        "PB_MQR_RANK": "市净率-MRQ排名",
        "PB_LYR": "市净率-LYR",
        "PB_LYR_RANK": "市净率-LYR排名",
        "PS_TTM": "市销率-TTM",
        "PS_TTM_RANK": "市销率-TTM排名",
        "PS_LYR": "市销率-LYR",
        "PS_LYR_RANK": "市销率-LYR排名",
        "PCE_TTM": "市现率-TTM",
        "PCE_TTM_RANK": "市现率-TTM排名",
        "PCE_LYR": "市现率-LYR",
        "PCE_LYR_RANK": "市现率-LYR排名",
    }
    temp_df = pd.DataFrame(columns=field_mapping.values())
    if data_json["result"] is not None:
        temp_df = pd.DataFrame(data_json["result"]["data"])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[field_mapping.values()]
    return temp_df


def stock_hk_scale_comparison_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-行业对比-规模对比
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/IndustryComparison
    :param symbol: 股票代码
    :type symbol: str
    :return: 规模对比
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "reportName": "RPT_PCF10_INDUSTRY_SCALE",
        "columns": "SECURITY_CODE,SECUCODE,TYPE_ID,TYPE_TYPE,TYPE_NAME,"
        "TYPE_NAME_EN,CORRE_SECURITY_CODE,CORRE_SECUCODE,"
        "CORRE_SECURITY_NAME,MAXSTDREPORTDATE,HKSDQMV,"
        "HKTOTAL_MARKET_CAP,OPERATE_INCOME,GROSS_PROFIT,"
        "HKSDQMV_RANK,HKTOTAL_CAP_RANK,OPERATE_INCOME_RANK,GROSS_PROFIT_RANK",
        "quoteColumns": "",
        "filter": f'(SECUCODE="{symbol}.HK")(CORRE_SECUCODE="{symbol}.HK")',
        "pageNumber": "1",
        "pageSize": "",
        "sortTypes": "",
        "sortColumns": "",
        "source": "F10",
        "client": "PC",
        "v": "07839693368708753",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    field_mapping = {
        "CORRE_SECURITY_CODE": "代码",
        "CORRE_SECURITY_NAME": "简称",
        "HKSDQMV": "总市值",
        "HKSDQMV_RANK": "总市值排名",
        "HKTOTAL_MARKET_CAP": "流通市值",
        "HKTOTAL_CAP_RANK": "流通市值排名",
        "OPERATE_INCOME": "营业总收入",
        "OPERATE_INCOME_RANK": "营业总收入排名",
        "GROSS_PROFIT": "净利润",
        "GROSS_PROFIT_RANK": "净利润排名",
    }
    temp_df = pd.DataFrame(columns=field_mapping.values())
    if data_json["result"] is not None:
        temp_df = pd.DataFrame(data_json["result"]["data"])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[field_mapping.values()]
    return temp_df


if __name__ == "__main__":
    stock_hk_growth_comparison_em_df = stock_hk_growth_comparison_em(symbol="03900")
    print(stock_hk_growth_comparison_em_df)

    stock_hk_valuation_comparison_em_df = stock_hk_valuation_comparison_em(
        symbol="03900"
    )
    print(stock_hk_valuation_comparison_em_df)

    stock_hk_scale_comparison_em_df = stock_hk_scale_comparison_em(symbol="03900")
    print(stock_hk_scale_comparison_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/27 14:20
Desc: 同花顺-板块-概念板块
https://q.10jqka.com.cn/thshy/
"""

from typing import Dict
from datetime import datetime
from functools import lru_cache
from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup
import py_mini_racer

from akshare.datasets import get_ths_js
from akshare.utils import demjson
from akshare.utils.tqdm import get_tqdm


def _get_file_content_ths(file: str = "ths.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


@lru_cache()
def _get_stock_board_concept_name_ths() -> dict:
    """
    获取同花顺概念板块代码和名称字典
    :return: 获取同花顺概念板块代码和名称字典
    :rtype: dict
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "https://q.10jqka.com.cn/gn/detail/code/307822/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    code_list = [
        item["href"].split("/")[-2]
        for item in soup.find(name="div", attrs={"class": "cate_inner"}).find_all("a")
    ]
    name_list = [
        item.text
        for item in soup.find(name="div", attrs={"class": "cate_inner"}).find_all("a")
    ]
    name_code_map = dict(zip(name_list, code_list))
    temp_dict = __stock_board_concept_summary_ths()
    name_code_map.update(temp_dict)
    return name_code_map


def stock_board_concept_name_ths() -> pd.DataFrame:
    """
    同花顺-板块-概念板块-概念
    http://q.10jqka.com.cn/thshy/
    :return: 所有概念板块的名称和链接
    :rtype: pandas.DataFrame
    """
    code_name_ths_map = _get_stock_board_concept_name_ths()
    temp_df = pd.DataFrame.from_dict(code_name_ths_map, orient="index")
    temp_df.reset_index(inplace=True)
    temp_df.columns = ["name", "code"]
    temp_df = temp_df[
        [
            "name",
            "code",
        ]
    ]
    return temp_df


def stock_board_concept_info_ths(symbol: str = "阿里巴巴概念") -> pd.DataFrame:
    """
    同花顺-板块-概念板块-板块简介
    http://q.10jqka.com.cn/gn/detail/code/301558/
    :param symbol: 板块简介
    :type symbol: str
    :return: 板块简介
    :rtype: pandas.DataFrame
    """
    stock_board_ths_map_df = stock_board_concept_name_ths()
    symbol_code = stock_board_ths_map_df[stock_board_ths_map_df["name"] == symbol][
        "code"
    ].values[0]
    url = f"http://q.10jqka.com.cn/gn/detail/code/{symbol_code}/"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    name_list = [
        item.text.strip()
        for item in soup.find(name="div", attrs={"class": "board-infos"}).find_all("dt")
    ]
    value_list = [
        item.text.strip().replace("\n", "/")
        for item in soup.find(name="div", attrs={"class": "board-infos"}).find_all("dd")
    ]
    temp_df = pd.DataFrame([name_list, value_list]).T
    temp_df.columns = ["项目", "值"]
    return temp_df


def stock_board_concept_index_ths(
    symbol: str = "阿里巴巴概念",
    start_date: str = "20200101",
    end_date: str = "20250228",
) -> pd.DataFrame:
    """
    同花顺-板块-概念板块-指数数据
    https://q.10jqka.com.cn/gn/detail/code/301558/
    :param start_date: 开始时间
    :type start_date: str
    :param end_date: 结束时间
    :type end_date: str
    :param symbol: 指数数据
    :type symbol: str
    :return: 指数数据
    :rtype: pandas.DataFrame
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")

    code_map = _get_stock_board_concept_name_ths()
    symbol_code = code_map[symbol]
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
    }
    url = f"https://q.10jqka.com.cn/gn/detail/code/{symbol_code}"
    r = requests.get(url=url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    inner_code = soup.find(name="input", attrs={"id": "clid"})["value"]
    big_df = pd.DataFrame()
    current_year = datetime.now().year
    begin_year = int(start_date[:4])
    tqdm = get_tqdm()
    for year in tqdm(range(begin_year, current_year + 1), leave=False):
        url = f"https://d.10jqka.com.cn/v4/line/bk_{inner_code}/01/{year}.js"
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/89.0.4389.90 Safari/537.36",
            "Referer": "http://q.10jqka.com.cn",
            "Host": "d.10jqka.com.cn",
            "Cookie": f"v={v_code}",
        }
        r = requests.get(url, headers=headers)
        data_text = r.text

        try:
            demjson.decode(data_text[data_text.find("{") : -1])
        except:  # noqa: E722
            continue
        temp_df = demjson.decode(data_text[data_text.find("{") : -1])
        temp_df = pd.DataFrame(temp_df["data"].split(";"))
        temp_df = temp_df.iloc[:, 0].str.split(",", expand=True)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    if len(big_df.columns) == 11:
        big_df.columns = [
            "日期",
            "开盘价",
            "最高价",
            "最低价",
            "收盘价",
            "成交量",
            "成交额",
            "_",
            "_",
            "_",
            "_",
        ]
    else:
        big_df.columns = [
            "日期",
            "开盘价",
            "最高价",
            "最低价",
            "收盘价",
            "成交量",
            "成交额",
            "_",
            "_",
            "_",
            "_",
            "_",
        ]
    big_df = big_df[
        [
            "日期",
            "开盘价",
            "最高价",
            "最低价",
            "收盘价",
            "成交量",
            "成交额",
        ]
    ]
    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    big_df.index = pd.to_datetime(big_df["日期"], errors="coerce")
    big_df = big_df[start_date:end_date]
    big_df.reset_index(drop=True, inplace=True)
    big_df["开盘价"] = pd.to_numeric(big_df["开盘价"], errors="coerce")
    big_df["最高价"] = pd.to_numeric(big_df["最高价"], errors="coerce")
    big_df["最低价"] = pd.to_numeric(big_df["最低价"], errors="coerce")
    big_df["收盘价"] = pd.to_numeric(big_df["收盘价"], errors="coerce")
    big_df["成交量"] = pd.to_numeric(big_df["成交量"], errors="coerce")
    big_df["成交额"] = pd.to_numeric(big_df["成交额"], errors="coerce")
    return big_df


@lru_cache()
def __stock_board_concept_summary_ths() -> Dict:
    """
    同花顺-数据中心-概念板块-概念时间表-辅助函数
    https://q.10jqka.com.cn/gn/
    :return: 概念时间表
    :rtype: dict
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "http://q.10jqka.com.cn/gn/index/field/addtime/order/desc/page/1/ajax/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    page_num = soup.find(name="span", attrs={"class": "page_info"}).text.split("/")[1]
    big_dict = dict()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(page_num) + 1), leave=False):
        url = f"http://q.10jqka.com.cn/gn/index/field/addtime/order/desc/page/{page}/ajax/1/"
        r = requests.get(url, headers=headers)
        try:
            soup = BeautifulSoup(r.text, features="lxml")
            temp_dict = {
                item.get_text(): item["href"].rsplit("/")[-2]
                for item in soup.find_all(name="a")
                if "detail" in item["href"]
            }
            big_dict.update(temp_dict)
        except ValueError:
            break
    return big_dict


def stock_board_concept_summary_ths() -> pd.DataFrame:
    """
    同花顺-数据中心-概念板块-概念时间表
    https://q.10jqka.com.cn/gn/
    :return: 概念时间表
    :rtype: pandas.DataFrame
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "http://q.10jqka.com.cn/gn/index/field/addtime/order/desc/page/1/ajax/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    page_num = soup.find(name="span", attrs={"class": "page_info"}).text.split("/")[1]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(page_num) + 1), leave=False):
        url = f"http://q.10jqka.com.cn/gn/index/field/addtime/order/desc/page/{page}/ajax/1/"
        r = requests.get(url, headers=headers)
        try:
            temp_df = pd.read_html(StringIO(r.text))[0]
            big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
        except ValueError:
            break
    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    big_df["成分股数量"] = pd.to_numeric(big_df["成分股数量"], errors="coerce")
    return big_df


if __name__ == "__main__":
    stock_board_concept_name_ths_df = stock_board_concept_name_ths()
    print(stock_board_concept_name_ths_df)

    stock_board_concept_info_ths_df = stock_board_concept_info_ths(
        symbol="阿里巴巴概念"
    )
    print(stock_board_concept_info_ths_df)

    stock_board_concept_index_ths_df = stock_board_concept_index_ths(
        symbol="DeepSeek概念", start_date="20200101", end_date="20250321"
    )
    print(stock_board_concept_index_ths_df)

    stock_board_concept_summary_ths_df = stock_board_concept_summary_ths()
    print(stock_board_concept_summary_ths_df)

    for stock in stock_board_concept_name_ths_df["name"]:
        print(stock)
        stock_board_industry_index_ths_df = stock_board_concept_index_ths(symbol=stock)
        print(stock_board_industry_index_ths_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/12/18 16:00
Desc: 同花顺-财务指标-主要指标
https://basic.10jqka.com.cn/new/000063/finance.html
"""

import json

import pandas as pd
import requests
from bs4 import BeautifulSoup

from akshare.utils.cons import headers


def stock_financial_abstract_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-主要指标
    https://basic.10jqka.com.cn/new/000063/finance.html
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "按年度", "按单季度"}
    :type indicator: str
    :return: 同花顺-财务指标-主要指标
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/new/{symbol}/finance.html"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    data_text = soup.find(name="p", attrs={"id": "main"}).string
    data_json = json.loads(data_text)
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    elif indicator == "按单季度":
        temp_df = pd.DataFrame(
            data_json["simple"][1:], columns=data_json["simple"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    temp_df.sort_values(by="报告期", ignore_index=True, inplace=True)
    return temp_df


def stock_financial_debt_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-资产负债表
    https://basic.10jqka.com.cn/new/000063/finance.html
    https://basic.10jqka.com.cn/api/stock/finance/000063_debt.json
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-资产负债表
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/api/stock/finance/{symbol}_debt.json"
    r = requests.get(url, headers=headers)
    data_json = json.loads(json.loads(r.text)["flashData"])
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


def stock_financial_benefit_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-利润表
    https://basic.10jqka.com.cn/new/000063/finance.html
    https://basic.10jqka.com.cn/api/stock/finance/000063_benefit.json
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期","按单季度", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-利润表
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/api/stock/finance/{symbol}_benefit.json"
    r = requests.get(url, headers=headers)
    data_json = json.loads(json.loads(r.text)["flashData"])
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    elif indicator == "按单季度":
        temp_df = pd.DataFrame(
            data_json["simple"][1:], columns=data_json["simple"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


def stock_financial_cash_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-现金流量表
    https://basic.10jqka.com.cn/new/000063/finance.html
    https://basic.10jqka.com.cn/api/stock/finance/000063_cash.json
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期","按单季度", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-现金流量表
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/api/stock/finance/{symbol}_cash.json"
    r = requests.get(url, headers=headers)
    data_json = json.loads(json.loads(r.text)["flashData"])
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    elif indicator == "按单季度":
        temp_df = pd.DataFrame(
            data_json["simple"][1:], columns=data_json["simple"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


def __get_market_code(stock_code: str = "000063") -> int:
    """
    同花顺-财务指标-主要指标-股票所属市场判断
    :param stock_code: 股票代码
    :type stock_code: str
    :return: 同花顺-财务指标-主要指标
    :rtype: pandas.DataFrame
    """
    # 确保股票代码是字符串并去掉空格
    stock_code = str(stock_code).strip()
    # 检查代码长度
    if len(stock_code) < 6:
        raise "请输入正确的股票代码"
    # 深交所股票: 000, 001, 002, 003, 300开头 (market代码33)
    if stock_code.startswith(("000", "001", "002", "003", "300")):
        return 33
    # 上交所股票: 600, 601, 603, 605, 688开头 (market代码17)
    if stock_code.startswith(("600", "601", "603", "605", "688")):
        return 17
    # 北交所股票: 920开头 (market代码151)
    if stock_code.startswith("920"):
        return 151
    # 其他情况无法识别
    return 0


def stock_financial_abstract_new_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-重要指标
    https://basic.10jqka.com.cn/new/000063/finance.html
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "一季度", "二季度", "三季度", "四季度", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-主要指标
    :rtype: pandas.DataFrame
    """
    url = "https://basic.10jqka.com.cn/basicapi/finance/index/v1/app_data/"
    if indicator == "按报告期":
        period = "0"
    elif indicator == "一季度":
        period = "1"
    elif indicator == "二季度":
        period = "2"
    elif indicator == "三季度":
        period = "3"
    elif indicator == "四季度":
        period = "4"
    else:
        period = "4"
    params = {
        "code": symbol,
        "id": "client_stock_importance",
        "market": __get_market_code(symbol),
        "type": "stock",
        "page": "1",
        "size": "50",
        "period": period,
    }
    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    # 提取财务数据
    financial_data = data_json["data"]["data"]
    # 创建空列表用于存储处理后的数据
    records = []
    # 收集所有可能的指标字段
    all_metric_fields = set()
    # 首先遍历一次，找出所有的字段名
    for report in financial_data:
        for metric_name, metric_values in report["index_list"].items():
            if isinstance(metric_values, dict):
                all_metric_fields.update(metric_values.keys())
    # 遍历每个报告期的数据
    for report in financial_data:
        report_date = report["date"]
        report_name = report["report_name"]
        report_period = report["report"]
        quarter_name = report["quarter_name"]
        # 遍历该报告期的所有财务指标
        for metric_name, metric_values in report["index_list"].items():
            # 基本信息
            record = {
                "report_date": report_date,
                "report_name": report_name,
                "report_period": report_period,
                "quarter_name": quarter_name,
                "metric_name": metric_name,
            }
            # 动态添加所有指标字段
            if isinstance(metric_values, dict):
                for field, value in metric_values.items():
                    record[field] = value
            else:
                # 如果不是字典，将其作为'value'字段
                record["value"] = metric_values
            records.append(record)
    # 创建DataFrame
    df = pd.DataFrame(records)
    # 自动识别并转换数值列
    numeric_columns = []
    for col in df.columns:
        if col not in [
            "report_date",
            "report_name",
            "report_period",
            "quarter_name",
            "metric_name",
        ]:
            # 尝试将列转换为数值类型
            if df[col].dtype == "object":
                # 替换空字符串为NaN
                df[col] = df[col].replace(to_replace="", value=pd.NA)
                # 尝试转换为数值
                numeric_series = pd.to_numeric(df[col], errors="coerce")
                # 如果大部分能转换为数值，则保留转换结果
                if numeric_series.notna().sum() > len(numeric_series) * 0.5:
                    df[col] = numeric_series
                    numeric_columns.append(col)
    return df


def stock_financial_debt_new_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-资产负债表
    https://basic.10jqka.com.cn/astockpc/astockmain/index.html#/financen?code=000063
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-资产负债表
    :rtype: pandas.DataFrame
    """
    url = "https://basic.10jqka.com.cn/basicapi/finance/index/v1/app_data/"
    if indicator == "按报告期":
        period = "0"
    else:
        period = "4"
    params = {
        "code": symbol,
        "id": "client_stock_debt",
        "market": __get_market_code(symbol),
        "type": "stock",
        "page": "1",
        "size": "50",
        "period": period,
    }
    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    # 提取财务数据
    financial_data = data_json["data"]["data"]
    # 创建空列表用于存储处理后的数据
    records = []
    # 收集所有可能的指标字段
    all_metric_fields = set()
    # 首先遍历一次，找出所有的字段名
    for report in financial_data:
        for metric_name, metric_values in report["index_list"].items():
            if isinstance(metric_values, dict):
                all_metric_fields.update(metric_values.keys())
    # 遍历每个报告期的数据
    for report in financial_data:
        report_date = report["date"]
        report_name = report["report_name"]
        report_period = report["report"]
        quarter_name = report["quarter_name"]
        # 遍历该报告期的所有财务指标
        for metric_name, metric_values in report["index_list"].items():
            # 基本信息
            record = {
                "report_date": report_date,
                "report_name": report_name,
                "report_period": report_period,
                "quarter_name": quarter_name,
                "metric_name": metric_name,
            }
            # 动态添加所有指标字段
            if isinstance(metric_values, dict):
                for field, value in metric_values.items():
                    record[field] = value
            else:
                # 如果不是字典，将其作为'value'字段
                record["value"] = metric_values
            records.append(record)
    # 创建DataFrame
    df = pd.DataFrame(records)
    # 自动识别并转换数值列
    numeric_columns = []
    for col in df.columns:
        if col not in [
            "report_date",
            "report_name",
            "report_period",
            "quarter_name",
            "metric_name",
        ]:
            # 尝试将列转换为数值类型
            if df[col].dtype == "object":
                # 替换空字符串为NaN
                df[col] = df[col].replace(to_replace="", value=pd.NA)
                # 尝试转换为数值
                numeric_series = pd.to_numeric(df[col], errors="coerce")
                # 如果大部分能转换为数值，则保留转换结果
                if numeric_series.notna().sum() > len(numeric_series) * 0.5:
                    df[col] = numeric_series
                    numeric_columns.append(col)
    return df


def stock_financial_benefit_new_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-利润表
    https://basic.10jqka.com.cn/astockpc/astockmain/index.html#/financen?code=000063
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "一季度", "二季度", "三季度", "四季度", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-利润表
    :rtype: pandas.DataFrame
    """
    url = "https://basic.10jqka.com.cn/basicapi/finance/index/v1/app_data/"
    if indicator == "按报告期":
        period = "0"
    elif indicator == "一季度":
        period = "1"
    elif indicator == "二季度":
        period = "2"
    elif indicator == "三季度":
        period = "3"
    elif indicator == "四季度":
        period = "4"
    else:
        period = "4"
    params = {
        "code": symbol,
        "id": "client_stock_benefit",
        "market": __get_market_code(symbol),
        "type": "stock",
        "page": "1",
        "size": "50",
        "period": period,
    }
    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    # 提取财务数据
    financial_data = data_json["data"]["data"]
    # 创建空列表用于存储处理后的数据
    records = []
    # 收集所有可能的指标字段
    all_metric_fields = set()
    # 首先遍历一次，找出所有的字段名
    for report in financial_data:
        for metric_name, metric_values in report["index_list"].items():
            if isinstance(metric_values, dict):
                all_metric_fields.update(metric_values.keys())
    # 遍历每个报告期的数据
    for report in financial_data:
        report_date = report["date"]
        report_name = report["report_name"]
        report_period = report["report"]
        quarter_name = report["quarter_name"]
        # 遍历该报告期的所有财务指标
        for metric_name, metric_values in report["index_list"].items():
            # 基本信息
            record = {
                "report_date": report_date,
                "report_name": report_name,
                "report_period": report_period,
                "quarter_name": quarter_name,
                "metric_name": metric_name,
            }
            # 动态添加所有指标字段
            if isinstance(metric_values, dict):
                for field, value in metric_values.items():
                    record[field] = value
            else:
                # 如果不是字典，将其作为'value'字段
                record["value"] = metric_values
            records.append(record)
    # 创建DataFrame
    df = pd.DataFrame(records)
    # 自动识别并转换数值列
    numeric_columns = []
    for col in df.columns:
        if col not in [
            "report_date",
            "report_name",
            "report_period",
            "quarter_name",
            "metric_name",
        ]:
            # 尝试将列转换为数值类型
            if df[col].dtype == "object":
                # 替换空字符串为NaN
                df[col] = df[col].replace(to_replace="", value=pd.NA)
                # 尝试转换为数值
                numeric_series = pd.to_numeric(df[col], errors="coerce")
                # 如果大部分能转换为数值，则保留转换结果
                if numeric_series.notna().sum() > len(numeric_series) * 0.5:
                    df[col] = numeric_series
                    numeric_columns.append(col)
    return df


def stock_financial_cash_new_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-现金流量表
    https://basic.10jqka.com.cn/astockpc/astockmain/index.html#/financen?code=000063
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "一季度", "二季度", "三季度", "四季度", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-现金流量表
    :rtype: pandas.DataFrame
    """
    url = "https://basic.10jqka.com.cn/basicapi/finance/index/v1/app_data/"
    if indicator == "按报告期":
        period = "0"
    elif indicator == "一季度":
        period = "1"
    elif indicator == "二季度":
        period = "2"
    elif indicator == "三季度":
        period = "3"
    elif indicator == "四季度":
        period = "4"
    else:
        period = "4"
    params = {
        "code": symbol,
        "id": "client_stock_cash",
        "market": __get_market_code(symbol),
        "type": "stock",
        "page": "1",
        "size": "50",
        "period": period,
    }
    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    # 提取财务数据
    financial_data = data_json["data"]["data"]
    # 创建空列表用于存储处理后的数据
    records = []
    # 收集所有可能的指标字段
    all_metric_fields = set()
    # 首先遍历一次，找出所有的字段名
    for report in financial_data:
        for metric_name, metric_values in report["index_list"].items():
            if isinstance(metric_values, dict):
                all_metric_fields.update(metric_values.keys())
    # 遍历每个报告期的数据
    for report in financial_data:
        report_date = report["date"]
        report_name = report["report_name"]
        report_period = report["report"]
        quarter_name = report["quarter_name"]
        # 遍历该报告期的所有财务指标
        for metric_name, metric_values in report["index_list"].items():
            # 基本信息
            record = {
                "report_date": report_date,
                "report_name": report_name,
                "report_period": report_period,
                "quarter_name": quarter_name,
                "metric_name": metric_name,
            }
            # 动态添加所有指标字段
            if isinstance(metric_values, dict):
                for field, value in metric_values.items():
                    record[field] = value
            else:
                # 如果不是字典，将其作为'value'字段
                record["value"] = metric_values
            records.append(record)
    # 创建DataFrame
    df = pd.DataFrame(records)
    # 自动识别并转换数值列
    numeric_columns = []
    for col in df.columns:
        if col not in [
            "report_date",
            "report_name",
            "report_period",
            "quarter_name",
            "metric_name",
        ]:
            # 尝试将列转换为数值类型
            if df[col].dtype == "object":
                # 替换空字符串为NaN
                df[col] = df[col].replace(to_replace="", value=pd.NA)
                # 尝试转换为数值
                numeric_series = pd.to_numeric(df[col], errors="coerce")
                # 如果大部分能转换为数值，则保留转换结果
                if numeric_series.notna().sum() > len(numeric_series) * 0.5:
                    df[col] = numeric_series
                    numeric_columns.append(col)
    return df


def stock_management_change_ths(symbol: str = "688981") -> pd.DataFrame:
    """
    同花顺-公司大事-高管持股变动
    https://basic.10jqka.com.cn/new/688981/event.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 同花顺-公司大事-高管持股变动
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/new/{symbol}/event.html"
    r = requests.get(url, headers=headers)
    r.encoding = "gb2312"
    soup = BeautifulSoup(r.text, features="lxml")
    soup_find = soup.find(name="table", attrs={"class": "data_table_1 m_table m_hl"})
    if soup_find is not None:
        content_list = [item.text.strip() for item in soup_find]
        column_names = content_list[1].split("\n")
        row = (
            content_list[3]
            .replace(" ", "")
            .replace("\t", "")
            .replace("\n\n", "")
            .replace("   ", "\n")
            .replace("\n\n", "\n")
            .split("\n")
        )
        row = [item for item in row if item != ""]
        new_rows = []
        step = len(column_names)
        for i in range(0, len(row), step):
            new_rows.append(row[i : i + step])
        temp_df = pd.DataFrame(new_rows, columns=column_names)
        temp_df.sort_values(by="变动日期", ignore_index=True, inplace=True)
        temp_df["变动日期"] = pd.to_datetime(
            temp_df["变动日期"], errors="coerce"
        ).dt.date
        temp_df.rename(
            columns={
                "变动数量（股）": "变动数量",
                "交易均价（元）": "交易均价",
                "剩余股数（股）": "剩余股数",
            },
            inplace=True,
        )
        return temp_df
    return pd.DataFrame()


def stock_shareholder_change_ths(symbol: str = "688981") -> pd.DataFrame:
    """
    同花顺-公司大事-股东持股变动
    https://basic.10jqka.com.cn/new/688981/event.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 同花顺-公司大事-股东持股变动
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/new/{symbol}/event.html"
    r = requests.get(url, headers=headers)
    r.encoding = "gb2312"
    soup = BeautifulSoup(r.text, features="lxml")
    soup_find = soup.find(name="table", attrs={"class": "m_table data_table_1 m_hl"})
    if soup_find is not None:
        content_list = [item.text.strip() for item in soup_find]
        column_names = content_list[1].split("\n")
        row = (
            content_list[3]
            .replace("\t", "")
            .replace("\n\n", "")
            .replace("   ", "\n")
            .replace(" ", "")
            .replace("\n\n", "\n")
            .split("\n")
        )
        row = [item for item in row if item != ""]
        new_rows = []
        step = len(column_names)
        for i in range(0, len(row), step):
            new_rows.append(row[i : i + step])
        temp_df = pd.DataFrame(new_rows, columns=column_names)
        temp_df.sort_values(by="公告日期", ignore_index=True, inplace=True)
        temp_df["公告日期"] = pd.to_datetime(
            temp_df["公告日期"], errors="coerce"
        ).dt.date
        temp_df.rename(
            columns={
                "变动数量(股)": "变动数量",
                "交易均价(元)": "交易均价",
                "剩余股份总数(股)": "剩余股份总数",
            },
            inplace=True,
        )
        return temp_df
    return pd.DataFrame()


if __name__ == "__main__":
    stock_financial_abstract_new_ths_df = stock_financial_abstract_new_ths(
        symbol="000063", indicator="按报告期"
    )
    print(stock_financial_abstract_new_ths_df)

    stock_financial_abstract_new_ths_df = stock_financial_abstract_new_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_abstract_new_ths_df)

    stock_financial_abstract_new_ths_df = stock_financial_abstract_new_ths(
        symbol="000063", indicator="一季度"
    )
    print(stock_financial_abstract_new_ths_df)

    stock_financial_debt_new_ths_df = stock_financial_debt_new_ths(
        symbol="002004", indicator="按报告期"
    )
    print(stock_financial_debt_new_ths_df)

    stock_financial_debt_new_ths_df = stock_financial_debt_new_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_debt_new_ths_df)

    stock_financial_benefit_new_ths_df = stock_financial_benefit_new_ths(
        symbol="000063", indicator="按报告期"
    )
    print(stock_financial_benefit_new_ths_df)

    stock_financial_benefit_new_ths_df = stock_financial_benefit_new_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_benefit_new_ths_df)

    stock_financial_benefit_new_ths_df = stock_financial_benefit_new_ths(
        symbol="000063", indicator="一季度"
    )
    print(stock_financial_benefit_new_ths_df)

    stock_financial_cash_new_ths_df = stock_financial_cash_new_ths(
        symbol="000063", indicator="按报告期"
    )
    print(stock_financial_cash_new_ths_df)

    stock_financial_cash_new_ths_df = stock_financial_cash_new_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_cash_new_ths_df)

    stock_financial_cash_new_ths_df = stock_financial_cash_new_ths(
        symbol="000063", indicator="一季度"
    )
    print(stock_financial_cash_new_ths_df)

    stock_management_change_ths_df = stock_management_change_ths(symbol="688981")
    print(stock_management_change_ths_df)

    stock_shareholder_change_ths_df = stock_shareholder_change_ths(symbol="688981")
    print(stock_shareholder_change_ths_df)

    stock_financial_abstract_ths_df = stock_financial_abstract_ths(symbol="000063", indicator="按报告期")
    print(stock_financial_abstract_ths_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/4/11 20:40
Desc: 全部A股-等权重市净率、中位数市净率
https://www.legulegu.com/stockdata/all-pb
"""

import pandas as pd
import requests

from akshare.stock_feature.stock_a_indicator import get_token_lg, get_cookie_csrf


def stock_a_all_pb() -> pd.DataFrame:
    """
    全部A股-等权重市净率、中位数市净率
    https://legulegu.com/stockdata/all-pb
    :return: 全部A股-等权重市盈率、中位数市盈率
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/api/stock-data/market-index-pb"
    params = {
        "marketId": "ALL",
        "token": get_token_lg(),
    }
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(url="https://legulegu.com/stockdata/all-pb"),
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df["date"] = (
        pd.to_datetime(temp_df["date"], utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    del temp_df["weightingAveragePB"]
    return temp_df


if __name__ == "__main__":
    stock_a_all_pb_df = stock_a_all_pb()
    print(stock_a_all_pb_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/4/22 14:00
Desc: 东方财富网-数据中心-龙虎榜单
https://data.eastmoney.com/stock/tradedetail.html
"""

import pandas as pd
import requests
from akshare.utils.tqdm import get_tqdm


def stock_lhb_detail_em(
    start_date: str = "20230403", end_date: str = "20230417"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-龙虎榜详情
    https://data.eastmoney.com/stock/tradedetail.html
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 龙虎榜详情
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "SECURITY_CODE,TRADE_DATE",
        "sortTypes": "1,-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_DAILYBILLBOARD_DETAILSNEW",
        "columns": "SECURITY_CODE,SECUCODE,SECURITY_NAME_ABBR,TRADE_DATE,EXPLAIN,CLOSE_PRICE,CHANGE_RATE,"
        "BILLBOARD_NET_AMT,BILLBOARD_BUY_AMT,BILLBOARD_SELL_AMT,BILLBOARD_DEAL_AMT,ACCUM_AMOUNT,"
        "DEAL_NET_RATIO,DEAL_AMOUNT_RATIO,TURNOVERRATE,FREE_MARKET_CAP,EXPLANATION,D1_CLOSE_ADJCHRATE,"
        "D2_CLOSE_ADJCHRATE,D5_CLOSE_ADJCHRATE,D10_CLOSE_ADJCHRATE,SECURITY_TYPE_CODE",
        "source": "WEB",
        "client": "WEB",
        "filter": f"(TRADE_DATE<='{end_date}')(TRADE_DATE>='{start_date}')",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page_num + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.rename(
        columns={
            "index": "序号",
            "SECURITY_CODE": "代码",
            "SECUCODE": "-",
            "SECURITY_NAME_ABBR": "名称",
            "TRADE_DATE": "上榜日",
            "EXPLAIN": "解读",
            "CLOSE_PRICE": "收盘价",
            "CHANGE_RATE": "涨跌幅",
            "BILLBOARD_NET_AMT": "龙虎榜净买额",
            "BILLBOARD_BUY_AMT": "龙虎榜买入额",
            "BILLBOARD_SELL_AMT": "龙虎榜卖出额",
            "BILLBOARD_DEAL_AMT": "龙虎榜成交额",
            "ACCUM_AMOUNT": "市场总成交额",
            "DEAL_NET_RATIO": "净买额占总成交比",
            "DEAL_AMOUNT_RATIO": "成交额占总成交比",
            "TURNOVERRATE": "换手率",
            "FREE_MARKET_CAP": "流通市值",
            "EXPLANATION": "上榜原因",
            "D1_CLOSE_ADJCHRATE": "上榜后1日",
            "D2_CLOSE_ADJCHRATE": "上榜后2日",
            "D5_CLOSE_ADJCHRATE": "上榜后5日",
            "D10_CLOSE_ADJCHRATE": "上榜后10日",
        },
        inplace=True,
    )

    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "上榜日",
            "解读",
            "收盘价",
            "涨跌幅",
            "龙虎榜净买额",
            "龙虎榜买入额",
            "龙虎榜卖出额",
            "龙虎榜成交额",
            "市场总成交额",
            "净买额占总成交比",
            "成交额占总成交比",
            "换手率",
            "流通市值",
            "上榜原因",
            "上榜后1日",
            "上榜后2日",
            "上榜后5日",
            "上榜后10日",
        ]
    ]
    big_df["上榜日"] = pd.to_datetime(big_df["上榜日"], errors="coerce").dt.date
    big_df["收盘价"] = pd.to_numeric(big_df["收盘价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["龙虎榜净买额"] = pd.to_numeric(big_df["龙虎榜净买额"], errors="coerce")
    big_df["龙虎榜买入额"] = pd.to_numeric(big_df["龙虎榜买入额"], errors="coerce")
    big_df["龙虎榜卖出额"] = pd.to_numeric(big_df["龙虎榜卖出额"], errors="coerce")
    big_df["龙虎榜成交额"] = pd.to_numeric(big_df["龙虎榜成交额"], errors="coerce")
    big_df["市场总成交额"] = pd.to_numeric(big_df["市场总成交额"], errors="coerce")
    big_df["净买额占总成交比"] = pd.to_numeric(
        big_df["净买额占总成交比"], errors="coerce"
    )
    big_df["成交额占总成交比"] = pd.to_numeric(
        big_df["成交额占总成交比"], errors="coerce"
    )
    big_df["换手率"] = pd.to_numeric(big_df["换手率"], errors="coerce")
    big_df["流通市值"] = pd.to_numeric(big_df["流通市值"], errors="coerce")
    big_df["上榜后1日"] = pd.to_numeric(big_df["上榜后1日"], errors="coerce")
    big_df["上榜后2日"] = pd.to_numeric(big_df["上榜后2日"], errors="coerce")
    big_df["上榜后5日"] = pd.to_numeric(big_df["上榜后5日"], errors="coerce")
    big_df["上榜后10日"] = pd.to_numeric(big_df["上榜后10日"], errors="coerce")
    return big_df


def stock_lhb_stock_statistic_em(symbol: str = "近一月") -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-个股上榜统计
    https://data.eastmoney.com/stock/tradedetail.html
    :param symbol: choice of {"近一月", "近三月", "近六月", "近一年"}
    :type symbol: str
    :return: 个股上榜统计
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "近一月": "01",
        "近三月": "02",
        "近六月": "03",
        "近一年": "04",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "BILLBOARD_TIMES,LATEST_TDATE,SECURITY_CODE",
        "sortTypes": "-1,-1,1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_BILLBOARD_TRADEALL",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f'(STATISTICS_CYCLE="{symbol_map[symbol]}")',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = [
        "序号",
        "-",
        "代码",
        "最近上榜日",
        "名称",
        "近1个月涨跌幅",
        "近3个月涨跌幅",
        "近6个月涨跌幅",
        "近1年涨跌幅",
        "涨跌幅",
        "收盘价",
        "-",
        "龙虎榜总成交额",
        "龙虎榜净买额",
        "-",
        "-",
        "机构买入净额",
        "上榜次数",
        "龙虎榜买入额",
        "龙虎榜卖出额",
        "机构买入总额",
        "机构卖出总额",
        "买方机构次数",
        "卖方机构次数",
        "-",
    ]
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最近上榜日",
            "收盘价",
            "涨跌幅",
            "上榜次数",
            "龙虎榜净买额",
            "龙虎榜买入额",
            "龙虎榜卖出额",
            "龙虎榜总成交额",
            "买方机构次数",
            "卖方机构次数",
            "机构买入净额",
            "机构买入总额",
            "机构卖出总额",
            "近1个月涨跌幅",
            "近3个月涨跌幅",
            "近6个月涨跌幅",
            "近1年涨跌幅",
        ]
    ]
    temp_df["最近上榜日"] = pd.to_datetime(
        temp_df["最近上榜日"], errors="coerce"
    ).dt.date
    return temp_df


def stock_lhb_jgmmtj_em(
    start_date: str = "20240417", end_date: str = "20240430"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-机构买卖每日统计
    https://data.eastmoney.com/stock/jgmmtj.html
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 机构买卖每日统计
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NET_BUY_AMT,TRADE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1,1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_ORGANIZATION_TRADE_DETAILS",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f"(TRADE_DATE>='{start_date}')(TRADE_DATE<='{end_date}')",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "-",
        "名称",
        "代码",
        "上榜日期",
        "收盘价",
        "涨跌幅",
        "买方机构数",
        "卖方机构数",
        "机构买入总额",
        "机构卖出总额",
        "机构买入净额",
        "市场总成交额",
        "机构净买额占总成交额比",
        "换手率",
        "流通市值",
        "上榜原因",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "收盘价",
            "涨跌幅",
            "买方机构数",
            "卖方机构数",
            "机构买入总额",
            "机构卖出总额",
            "机构买入净额",
            "市场总成交额",
            "机构净买额占总成交额比",
            "换手率",
            "流通市值",
            "上榜原因",
            "上榜日期",
        ]
    ]
    big_df["上榜日期"] = pd.to_datetime(big_df["上榜日期"], errors="coerce").dt.date
    big_df["收盘价"] = pd.to_numeric(big_df["收盘价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["买方机构数"] = pd.to_numeric(big_df["买方机构数"], errors="coerce")
    big_df["卖方机构数"] = pd.to_numeric(big_df["卖方机构数"], errors="coerce")
    big_df["机构买入总额"] = pd.to_numeric(big_df["机构买入总额"], errors="coerce")
    big_df["机构卖出总额"] = pd.to_numeric(big_df["机构卖出总额"], errors="coerce")
    big_df["机构买入净额"] = pd.to_numeric(big_df["机构买入净额"], errors="coerce")
    big_df["市场总成交额"] = pd.to_numeric(big_df["市场总成交额"], errors="coerce")
    big_df["机构净买额占总成交额比"] = pd.to_numeric(
        big_df["机构净买额占总成交额比"], errors="coerce"
    )
    big_df["换手率"] = pd.to_numeric(big_df["换手率"], errors="coerce")
    big_df["流通市值"] = pd.to_numeric(big_df["流通市值"], errors="coerce")
    return big_df


def stock_lhb_jgstatistic_em(symbol: str = "近一月") -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-机构席位追踪
    https://data.eastmoney.com/stock/jgstatistic.html
    :param symbol: choice of {"近一月", "近三月", "近六月", "近一年"}
    :type symbol: str
    :return: 机构席位追踪
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "近一月": "01",
        "近三月": "02",
        "近六月": "03",
        "近一年": "04",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "ONLIST_TIMES,SECURITY_CODE",
        "sortTypes": "-1,1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_ORGANIZATION_SEATNEW",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f'(STATISTICSCYCLE="{symbol_map[symbol]}")',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.rename(
        columns={
            "index": "序号",
            "SECURITY_CODE": "代码",
            "SECURITY_NAME_ABBR": "名称",
            "CLOSE_PRICE": "收盘价",
            "CHANGE_RATE": "涨跌幅",
            "AMOUNT": "龙虎榜成交金额",
            "ONLIST_TIMES": "上榜次数",
            "BUY_AMT": "机构买入额",
            "BUY_TIMES": "机构买入次数",
            "SELL_AMT": "机构卖出额",
            "SELL_TIMES": "机构卖出次数",
            "NET_BUY_AMT": "机构净买额",
            "M1_CLOSE_ADJCHRATE": "近1个月涨跌幅",
            "M3_CLOSE_ADJCHRATE": "近3个月涨跌幅",
            "M6_CLOSE_ADJCHRATE": "近6个月涨跌幅",
            "Y1_CLOSE_ADJCHRATE": "近1年涨跌幅",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "收盘价",
            "涨跌幅",
            "龙虎榜成交金额",
            "上榜次数",
            "机构买入额",
            "机构买入次数",
            "机构卖出额",
            "机构卖出次数",
            "机构净买额",
            "近1个月涨跌幅",
            "近3个月涨跌幅",
            "近6个月涨跌幅",
            "近1年涨跌幅",
        ]
    ]

    big_df["收盘价"] = pd.to_numeric(big_df["收盘价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["龙虎榜成交金额"] = pd.to_numeric(big_df["龙虎榜成交金额"], errors="coerce")
    big_df["上榜次数"] = pd.to_numeric(big_df["上榜次数"], errors="coerce")
    big_df["机构买入额"] = pd.to_numeric(big_df["机构买入额"], errors="coerce")
    big_df["机构买入次数"] = pd.to_numeric(big_df["机构买入次数"], errors="coerce")
    big_df["机构卖出额"] = pd.to_numeric(big_df["机构卖出额"], errors="coerce")
    big_df["机构卖出次数"] = pd.to_numeric(big_df["机构卖出次数"], errors="coerce")
    big_df["机构净买额"] = pd.to_numeric(big_df["机构净买额"], errors="coerce")
    big_df["近1个月涨跌幅"] = pd.to_numeric(big_df["近1个月涨跌幅"], errors="coerce")
    big_df["近3个月涨跌幅"] = pd.to_numeric(big_df["近3个月涨跌幅"], errors="coerce")
    big_df["近6个月涨跌幅"] = pd.to_numeric(big_df["近6个月涨跌幅"], errors="coerce")
    big_df["近1年涨跌幅"] = pd.to_numeric(big_df["近1年涨跌幅"], errors="coerce")
    return big_df


def stock_lhb_hyyyb_em(
    start_date: str = "20220324", end_date: str = "20220324"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-每日活跃营业部
    https://data.eastmoney.com/stock/hyyyb.html
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 每日活跃营业部
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TOTAL_NETAMT,ONLIST_DATE,OPERATEDEPT_CODE",
        "sortTypes": "-1,-1,1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_OPERATEDEPT_ACTIVE",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f"(ONLIST_DATE>='{start_date}')(ONLIST_DATE<='{end_date}')",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "营业部名称",
        "上榜日",
        "买入个股数",
        "卖出个股数",
        "买入总金额",
        "卖出总金额",
        "总买卖净额",
        "-",
        "营业部代码",
        "买入股票",
        "-",
        "-",
    ]
    big_df = big_df[
        [
            "序号",
            "营业部名称",
            "上榜日",
            "买入个股数",
            "卖出个股数",
            "买入总金额",
            "卖出总金额",
            "总买卖净额",
            "买入股票",
            "营业部代码",
        ]
    ]

    big_df["上榜日"] = pd.to_datetime(big_df["上榜日"], errors="coerce").dt.date
    big_df["买入个股数"] = pd.to_numeric(big_df["买入个股数"], errors="coerce")
    big_df["卖出个股数"] = pd.to_numeric(big_df["卖出个股数"], errors="coerce")
    big_df["买入总金额"] = pd.to_numeric(big_df["买入总金额"], errors="coerce")
    big_df["卖出总金额"] = pd.to_numeric(big_df["卖出总金额"], errors="coerce")
    big_df["总买卖净额"] = pd.to_numeric(big_df["总买卖净额"], errors="coerce")
    return big_df


def stock_lhb_yybph_em(symbol: str = "近一月") -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-营业部排行
    https://data.eastmoney.com/stock/yybph.html
    :param symbol: choice of {"近一月", "近三月", "近六月", "近一年"}
    :type symbol: str
    :return: 营业部排行
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "近一月": "01",
        "近三月": "02",
        "近六月": "03",
        "近一年": "04",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TOTAL_BUYER_SALESTIMES_1DAY,OPERATEDEPT_CODE",
        "sortTypes": "-1,1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_RATEDEPT_RETURNT_RANKING",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f'(STATISTICSCYCLE="{symbol_map[symbol]}")',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.rename(
        columns={
            "index": "序号",
            "OPERATEDEPT_NAME": "营业部名称",
            "TOTAL_BUYER_SALESTIMES_1DAY": "上榜后1天-买入次数",
            "AVERAGE_INCREASE_1DAY": "上榜后1天-平均涨幅",
            "RISE_PROBABILITY_1DAY": "上榜后1天-上涨概率",
            "TOTAL_BUYER_SALESTIMES_2DAY": "上榜后2天-买入次数",
            "AVERAGE_INCREASE_2DAY": "上榜后2天-平均涨幅",
            "RISE_PROBABILITY_2DAY": "上榜后2天-上涨概率",
            "TOTAL_BUYER_SALESTIMES_3DAY": "上榜后3天-买入次数",
            "AVERAGE_INCREASE_3DAY": "上榜后3天-平均涨幅",
            "RISE_PROBABILITY_3DAY": "上榜后3天-上涨概率",
            "TOTAL_BUYER_SALESTIMES_5DAY": "上榜后5天-买入次数",
            "AVERAGE_INCREASE_5DAY": "上榜后5天-平均涨幅",
            "RISE_PROBABILITY_5DAY": "上榜后5天-上涨概率",
            "TOTAL_BUYER_SALESTIMES_10DAY": "上榜后10天-买入次数",
            "AVERAGE_INCREASE_10DAY": "上榜后10天-平均涨幅",
            "RISE_PROBABILITY_10DAY": "上榜后10天-上涨概率",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "序号",
            "营业部名称",
            "上榜后1天-买入次数",
            "上榜后1天-平均涨幅",
            "上榜后1天-上涨概率",
            "上榜后2天-买入次数",
            "上榜后2天-平均涨幅",
            "上榜后2天-上涨概率",
            "上榜后3天-买入次数",
            "上榜后3天-平均涨幅",
            "上榜后3天-上涨概率",
            "上榜后5天-买入次数",
            "上榜后5天-平均涨幅",
            "上榜后5天-上涨概率",
            "上榜后10天-买入次数",
            "上榜后10天-平均涨幅",
            "上榜后10天-上涨概率",
        ]
    ]

    big_df["上榜后1天-买入次数"] = pd.to_numeric(
        big_df["上榜后1天-买入次数"], errors="coerce"
    )
    big_df["上榜后1天-平均涨幅"] = pd.to_numeric(
        big_df["上榜后1天-平均涨幅"], errors="coerce"
    )
    big_df["上榜后1天-上涨概率"] = pd.to_numeric(
        big_df["上榜后1天-上涨概率"], errors="coerce"
    )

    big_df["上榜后2天-买入次数"] = pd.to_numeric(
        big_df["上榜后2天-买入次数"], errors="coerce"
    )
    big_df["上榜后2天-平均涨幅"] = pd.to_numeric(
        big_df["上榜后2天-平均涨幅"], errors="coerce"
    )
    big_df["上榜后2天-上涨概率"] = pd.to_numeric(
        big_df["上榜后2天-上涨概率"], errors="coerce"
    )

    big_df["上榜后3天-买入次数"] = pd.to_numeric(
        big_df["上榜后3天-买入次数"], errors="coerce"
    )
    big_df["上榜后3天-平均涨幅"] = pd.to_numeric(
        big_df["上榜后3天-平均涨幅"], errors="coerce"
    )
    big_df["上榜后3天-上涨概率"] = pd.to_numeric(
        big_df["上榜后3天-上涨概率"], errors="coerce"
    )

    big_df["上榜后5天-买入次数"] = pd.to_numeric(
        big_df["上榜后5天-买入次数"], errors="coerce"
    )
    big_df["上榜后5天-平均涨幅"] = pd.to_numeric(
        big_df["上榜后5天-平均涨幅"], errors="coerce"
    )
    big_df["上榜后5天-上涨概率"] = pd.to_numeric(
        big_df["上榜后5天-上涨概率"], errors="coerce"
    )

    big_df["上榜后10天-买入次数"] = pd.to_numeric(
        big_df["上榜后10天-买入次数"], errors="coerce"
    )
    big_df["上榜后10天-平均涨幅"] = pd.to_numeric(
        big_df["上榜后10天-平均涨幅"], errors="coerce"
    )
    big_df["上榜后10天-上涨概率"] = pd.to_numeric(
        big_df["上榜后10天-上涨概率"], errors="coerce"
    )
    return big_df


def stock_lhb_traderstatistic_em(symbol: str = "近一月") -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-营业部统计
    https://data.eastmoney.com/stock/traderstatistic.html
    :param symbol: choice of {"近一月", "近三月", "近六月", "近一年"}
    :type symbol: str
    :return: 营业部统计
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "近一月": "01",
        "近三月": "02",
        "近六月": "03",
        "近一年": "04",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "AMOUNT,OPERATEDEPT_CODE",
        "sortTypes": "-1,1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_OPERATEDEPT_LIST_STATISTICS",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f'(STATISTICSCYCLE="{symbol_map[symbol]}")',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.rename(
        columns={
            "index": "序号",
            "OPERATEDEPT_NAME": "营业部名称",
            "AMOUNT": "龙虎榜成交金额",
            "SALES_ONLIST_TIMES": "上榜次数",
            "ACT_BUY": "买入额",
            "TOTAL_BUYER_SALESTIMES": "买入次数",
            "ACT_SELL": "卖出额",
            "TOTAL_SELLER_SALESTIMES": "卖出次数",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "序号",
            "营业部名称",
            "龙虎榜成交金额",
            "上榜次数",
            "买入额",
            "买入次数",
            "卖出额",
            "卖出次数",
        ]
    ]

    big_df["龙虎榜成交金额"] = pd.to_numeric(big_df["龙虎榜成交金额"], errors="coerce")
    big_df["上榜次数"] = pd.to_numeric(big_df["上榜次数"], errors="coerce")
    big_df["买入额"] = pd.to_numeric(big_df["买入额"], errors="coerce")
    big_df["买入次数"] = pd.to_numeric(big_df["买入次数"], errors="coerce")
    big_df["卖出额"] = pd.to_numeric(big_df["卖出额"], errors="coerce")
    big_df["卖出次数"] = pd.to_numeric(big_df["卖出次数"], errors="coerce")
    return big_df


def stock_lhb_stock_detail_date_em(symbol: str = "600077") -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-个股龙虎榜详情-日期
    https://data.eastmoney.com/stock/tradedetail.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 个股龙虎榜详情-日期
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_LHB_BOARDDATE",
        "columns": "SECURITY_CODE,TRADE_DATE,TR_DATE",
        "filter": f'(SECURITY_CODE="{symbol}")',
        "pageNumber": "1",
        "pageSize": "1000",
        "sortTypes": "-1",
        "sortColumns": "TRADE_DATE",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = [
        "序号",
        "股票代码",
        "交易日",
        "-",
    ]
    temp_df = temp_df[
        [
            "序号",
            "股票代码",
            "交易日",
        ]
    ]
    temp_df["交易日"] = pd.to_datetime(temp_df["交易日"]).dt.date
    return temp_df


def stock_lhb_stock_detail_em(
    symbol: str = "000788", date: str = "20220315", flag: str = "卖出"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-个股龙虎榜详情
    https://data.eastmoney.com/stock/lhb/600077.html
    :param symbol: 股票代码
    :type symbol: str
    :param date: 查询日期; 需要通过 ak.stock_lhb_stock_detail_date_em(symbol="600077") 接口获取相应股票的有龙虎榜详情数据的日期
    :type date: str
    :param flag: choice of {"买入", "卖出"}
    :type flag: str
    :return: 个股龙虎榜详情
    :rtype: pandas.DataFrame
    """
    flag_map = {
        "买入": "BUY",
        "卖出": "SELL",
    }
    report_map = {
        "买入": "RPT_BILLBOARD_DAILYDETAILSBUY",
        "卖出": "RPT_BILLBOARD_DAILYDETAILSSELL",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": report_map[flag],
        "columns": "ALL",
        "filter": f"""(TRADE_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')(SECURITY_CODE="{symbol}")""",
        "pageNumber": "1",
        "pageSize": "500",
        "sortTypes": "-1",
        "sortColumns": flag_map[flag],
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1

    if flag == "买入":
        temp_df.columns = [
            "序号",
            "-",
            "-",
            "-",
            "-",
            "交易营业部名称",
            "类型",
            "-",
            "-",
            "-",
            "-",
            "买入金额",
            "卖出金额",
            "净额",
            "-",
            "-",
            "-",
            "-",
            "买入金额-占总成交比例",
            "卖出金额-占总成交比例",
            "-",
        ]
        temp_df = temp_df[
            [
                "序号",
                "交易营业部名称",
                "买入金额",
                "买入金额-占总成交比例",
                "卖出金额",
                "卖出金额-占总成交比例",
                "净额",
                "类型",
            ]
        ]
        temp_df["买入金额"] = pd.to_numeric(temp_df["买入金额"], errors="coerce")
        temp_df["买入金额-占总成交比例"] = pd.to_numeric(
            temp_df["买入金额-占总成交比例"], errors="coerce"
        )
        temp_df["卖出金额"] = pd.to_numeric(temp_df["卖出金额"], errors="coerce")
        temp_df["卖出金额-占总成交比例"] = pd.to_numeric(
            temp_df["卖出金额-占总成交比例"], errors="coerce"
        )
        temp_df.sort_values("类型", inplace=True, ignore_index=True)
        temp_df.reset_index(inplace=True, drop=True)
        temp_df["序号"] = range(1, len(temp_df["序号"]) + 1)
    else:
        temp_df.columns = [
            "序号",
            "-",
            "-",
            "-",
            "-",
            "交易营业部名称",
            "类型",
            "-",
            "-",
            "-",
            "-",
            "买入金额",
            "卖出金额",
            "净额",
            "-",
            "-",
            "-",
            "-",
            "买入金额-占总成交比例",
            "卖出金额-占总成交比例",
            "-",
        ]
        temp_df = temp_df[
            [
                "序号",
                "交易营业部名称",
                "买入金额",
                "买入金额-占总成交比例",
                "卖出金额",
                "卖出金额-占总成交比例",
                "净额",
                "类型",
            ]
        ]
        temp_df["买入金额"] = pd.to_numeric(temp_df["买入金额"], errors="coerce")
        temp_df["买入金额-占总成交比例"] = pd.to_numeric(
            temp_df["买入金额-占总成交比例"], errors="coerce"
        )
        temp_df["卖出金额"] = pd.to_numeric(temp_df["卖出金额"], errors="coerce")
        temp_df["卖出金额-占总成交比例"] = pd.to_numeric(
            temp_df["卖出金额-占总成交比例"], errors="coerce"
        )
        temp_df.sort_values("类型", inplace=True, ignore_index=True)
        temp_df.reset_index(inplace=True, drop=True)
        temp_df["序号"] = range(1, len(temp_df["序号"]) + 1)
    return temp_df


def stock_lhb_yyb_detail_em(symbol: str = "10188715") -> pd.DataFrame:
    """
    东方财富网-数据中心-龙虎榜单-营业部历史交易明细-营业部交易明细
    https://data.eastmoney.com/stock/lhb/yyb/10188715.html
    :param symbol: 营业部代码, 如 "10188715", 通过 ak.stock_lhb_hyyyb_em() 接口获取
    :type symbol: str
    :return: 营业部交易明细数据
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TRADE_DATE,SECURITY_CODE",
        "sortTypes": "-1,1",
        "pageSize": "100",
        "pageNumber": "1",
        "reportName": "RPT_OPERATEDEPT_TRADE_DETAILSNEW",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f'(OPERATEDEPT_CODE="{symbol}")',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    # 检查DataFrame是否为空
    if big_df.empty:
        return pd.DataFrame()

    # 确保列名与实际返回的JSON数据结构一致
    column_map = {
        "OPERATEDEPT_CODE": "营业部代码",
        "OPERATEDEPT_NAME": "营业部名称",
        "TRADE_DATE": "交易日期",
        "D1_CLOSE_ADJCHRATE": "1日后涨跌幅",
        "D2_CLOSE_ADJCHRATE": "2日后涨跌幅",
        "D3_CLOSE_ADJCHRATE": "3日后涨跌幅",
        "D5_CLOSE_ADJCHRATE": "5日后涨跌幅",
        "D10_CLOSE_ADJCHRATE": "10日后涨跌幅",
        "SECURITY_CODE": "股票代码",
        "SECURITY_NAME_ABBR": "股票名称",
        "ACT_BUY": "买入金额",
        "ACT_SELL": "卖出金额",
        "NET_AMT": "净额",
        "EXPLANATION": "上榜原因",
        "D20_CLOSE_ADJCHRATE": "20日后涨跌幅",
        "D30_CLOSE_ADJCHRATE": "30日后涨跌幅",
        "SECUCODE": "证券代码",
        "OPERATEDEPT_CODE_OLD": "营业部旧代码",
        "ORG_NAME_ABBR": "营业部简称",
        "CHANGE_RATE": "涨跌幅",
    }

    # 重命名列
    big_df.rename(columns=column_map, inplace=True)

    # 添加序号列
    big_df.reset_index(inplace=True)
    big_df["序号"] = big_df.index + 1

    # 选择需要的列并排序
    result_columns = [
        "序号",
        "营业部代码",
        "营业部名称",
        "营业部简称",
        "交易日期",
        "股票代码",
        "股票名称",
        "涨跌幅",
        "买入金额",
        "卖出金额",
        "净额",
        "上榜原因",
        "1日后涨跌幅",
        "2日后涨跌幅",
        "3日后涨跌幅",
        "5日后涨跌幅",
        "10日后涨跌幅",
        "20日后涨跌幅",
        "30日后涨跌幅",
    ]

    # 确保所有列都存在
    for col in result_columns:
        if col not in big_df.columns and col != "序号":
            big_df[col] = None

    big_df = big_df[result_columns]

    # 处理日期格式
    big_df["交易日期"] = pd.to_datetime(big_df["交易日期"], errors="coerce").dt.date

    # 处理数值列
    numeric_cols = [
        "涨跌幅",
        "买入金额",
        "卖出金额",
        "净额",
        "1日后涨跌幅",
        "2日后涨跌幅",
        "3日后涨跌幅",
        "5日后涨跌幅",
        "10日后涨跌幅",
        "20日后涨跌幅",
        "30日后涨跌幅",
    ]
    for col in numeric_cols:
        big_df[col] = pd.to_numeric(big_df[col], errors="coerce")

    return big_df


if __name__ == "__main__":
    stock_lhb_detail_em_df = stock_lhb_detail_em(
        start_date="20250201", end_date="20250228"
    )
    print(stock_lhb_detail_em_df)

    stock_lhb_stock_statistic_em_df = stock_lhb_stock_statistic_em(symbol="近一月")
    print(stock_lhb_stock_statistic_em_df)

    stock_lhb_stock_statistic_em_df = stock_lhb_stock_statistic_em(symbol="近三月")
    print(stock_lhb_stock_statistic_em_df)

    stock_lhb_stock_statistic_em_df = stock_lhb_stock_statistic_em(symbol="近六月")
    print(stock_lhb_stock_statistic_em_df)

    stock_lhb_stock_statistic_em_df = stock_lhb_stock_statistic_em(symbol="近一年")
    print(stock_lhb_stock_statistic_em_df)

    stock_lhb_jgmmtj_em_df = stock_lhb_jgmmtj_em(
        start_date="20240417", end_date="20240430"
    )
    print(stock_lhb_jgmmtj_em_df)

    stock_lhb_jgstatistic_em_df = stock_lhb_jgstatistic_em(symbol="近一月")
    print(stock_lhb_jgstatistic_em_df)

    stock_lhb_hyyyb_em_df = stock_lhb_hyyyb_em(
        start_date="20240401", end_date="20240430"
    )
    print(stock_lhb_hyyyb_em_df)

    stock_lhb_yybph_em_df = stock_lhb_yybph_em(symbol="近一月")
    print(stock_lhb_yybph_em_df)

    stock_lhb_traderstatistic_em_df = stock_lhb_traderstatistic_em(symbol="近一月")
    print(stock_lhb_traderstatistic_em_df)

    stock_lhb_stock_detail_date_em_df = stock_lhb_stock_detail_date_em(symbol="002901")
    print(stock_lhb_stock_detail_date_em_df)

    stock_lhb_stock_detail_em_df = stock_lhb_stock_detail_em(
        symbol="002901", date="20221012", flag="买入"
    )
    print(stock_lhb_stock_detail_em_df)

    stock_lhb_stock_detail_em_df = stock_lhb_stock_detail_em(
        symbol="600077", date="20070416", flag="买入"
    )
    print(stock_lhb_stock_detail_em_df)

    stock_lhb_yyb_detail_em_df = stock_lhb_yyb_detail_em(symbol="10188715")
    print(stock_lhb_yyb_detail_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/6/16 18:19
Desc: 百度股市通-热搜股票
https://gushitong.baidu.com/hotlist?mainTab=hotSearch&market=all
"""

from datetime import datetime

import pandas as pd
import requests


def stock_hot_search_baidu(
    symbol: str = "A股", date: str = "20250616", time: str = "今日"
):
    """
    百度股市通-热搜股票
    https://gushitong.baidu.com/hotlist?mainTab=hotSearch&market=all
    :param symbol: choice of {"全部", "A股", "港股", "美股"}
    :type symbol: str
    :param date: 日期
    :type date: str
    :param time: time="今日"；choice of {"今日", "1小时"}
    :type time: str
    :return: 热搜股票
    :rtype: pandas.DataFrame
    """
    hour_str = datetime.now().hour
    symbol_map = {
        "全市场": "all",
        "A股": "ab",
        "港股": "hk",
        "美股": "us",
    }
    url = "https://finance.pae.baidu.com/selfselect/listsugrecomm"
    params = {
        "bizType": "wisexmlnew",
        "dsp": "iphone",
        "product": "search",
        "style": "tablelist",
        "market": symbol_map[symbol],
        "type": time,
        "day": date,
        "hour": hour_str,
        "pn": "0",
        "rn": "12",
        "finClientType": "pc",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["Result"]["list"]["body"])
    temp_df.rename(
        columns={
            "name": "名称/代码",
            "pxChangeRate": "涨跌幅",
            "heat": "综合热度",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "名称/代码",
            "涨跌幅",
            "综合热度",
        ]
    ]
    temp_df["综合热度"] = pd.to_numeric(temp_df["综合热度"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_hot_search_baidu_df = stock_hot_search_baidu(
        symbol="A股", date="20250616", time="今日"
    )
    print(stock_hot_search_baidu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/16 15:30
Desc: 同花顺-港股-分红派息
https://stockpage.10jqka.com.cn/HK0700/bonus/
"""

from io import StringIO

import pandas as pd
import requests


def stock_hk_fhpx_detail_ths(symbol: str = "0700") -> pd.DataFrame:
    """
    同花顺-港股-分红派息
    https://stockpage.10jqka.com.cn/HK0700/bonus/
    :param symbol: 港股代码
    :type symbol: str
    :return: 分红派息
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/176/HK{symbol}/bonus.html"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    r.encoding = "utf-8"
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df.columns = [
        "公告日期",
        "方案",
        "除净日",
        "派息日",
        "过户日期起止日-起始",
        "过户日期起止日-截止",
        "类型",
        "进度",
        "以股代息",
    ]
    # 剔除异常格式，由以股代息产生的异常
    temp_df.dropna(subset=["派息日", "除净日"], inplace=True, ignore_index=True)
    temp_df["公告日期"] = pd.to_datetime(
        temp_df["公告日期"], format="%Y-%m-%d", errors="coerce"
    ).dt.date
    temp_df["除净日"] = pd.to_datetime(
        temp_df["除净日"], format="%Y-%m-%d", errors="coerce"
    ).dt.date
    temp_df["派息日"] = pd.to_datetime(
        temp_df["派息日"], format="%Y-%m-%d", errors="coerce"
    ).dt.date
    temp_df["过户日期起止日-起始"] = pd.to_datetime(
        temp_df["过户日期起止日-起始"], format="%Y-%m-%d", errors="coerce"
    ).dt.date
    temp_df["过户日期起止日-截止"] = pd.to_datetime(
        temp_df["过户日期起止日-截止"], format="%Y-%m-%d", errors="coerce"
    ).dt.date
    temp_df.sort_values(["公告日期"], inplace=True, ignore_index=True)
    return temp_df


if __name__ == "__main__":
    stock_hk_fhpx_detail_ths_df = stock_hk_fhpx_detail_ths(symbol="0700")
    print(stock_hk_fhpx_detail_ths_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/9/20 15:30
Desc: 东方财富-数据中心-年报季报-业绩快报-三大报表
资产负债表
https://data.eastmoney.com/bbsj/202003/zcfz.html
利润表
https://data.eastmoney.com/bbsj/202003/lrb.html
现金流量表
https://data.eastmoney.com/bbsj/202003/xjll.html
"""

import pandas as pd
import requests

from akshare.utils.tqdm import get_tqdm


def stock_zcfz_em(date: str = "20240331") -> pd.DataFrame:
    """
    东方财富-数据中心-年报季报-业绩快报-资产负债表
    https://data.eastmoney.com/bbsj/202003/zcfz.html
    :param date: choice of {"20200331", "20200630", "20200930", "20201231", "..."}; 从 20100331 开始
    :type date: str
    :return: 资产负债表
    :rtype: pandas.DataFrame
    """
    import warnings

    warnings.filterwarnings(action="ignore", category=FutureWarning)
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_DMSK_FN_BALANCE",
        "columns": "ALL",
        "filter": f"""(SECURITY_TYPE_CODE in ("058001001","058001008"))(TRADE_MARKET_CODE!="069001017")
        (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], join="outer", ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "_",
        "股票代码",
        "_",
        "_",
        "股票简称",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "公告日期",
        "_",
        "资产-总资产",
        "_",
        "资产-货币资金",
        "_",
        "资产-应收账款",
        "_",
        "资产-存货",
        "_",
        "负债-总负债",
        "负债-应付账款",
        "_",
        "负债-预收账款",
        "_",
        "股东权益合计",
        "_",
        "资产-总资产同比",
        "负债-总负债同比",
        "_",
        "资产负债率",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "资产-货币资金",
            "资产-应收账款",
            "资产-存货",
            "资产-总资产",
            "资产-总资产同比",
            "负债-应付账款",
            "负债-预收账款",
            "负债-总负债",
            "负债-总负债同比",
            "资产负债率",
            "股东权益合计",
            "公告日期",
        ]
    ]

    big_df["资产-货币资金"] = pd.to_numeric(big_df["资产-货币资金"], errors="coerce")
    big_df["资产-应收账款"] = pd.to_numeric(big_df["资产-应收账款"], errors="coerce")
    big_df["资产-存货"] = pd.to_numeric(big_df["资产-存货"], errors="coerce")
    big_df["资产-总资产"] = pd.to_numeric(big_df["资产-总资产"], errors="coerce")
    big_df["资产-总资产同比"] = pd.to_numeric(
        big_df["资产-总资产同比"], errors="coerce"
    )
    big_df["负债-应付账款"] = pd.to_numeric(big_df["负债-应付账款"], errors="coerce")
    big_df["负债-预收账款"] = pd.to_numeric(big_df["负债-预收账款"], errors="coerce")
    big_df["负债-总负债"] = pd.to_numeric(big_df["负债-总负债"], errors="coerce")
    big_df["负债-总负债同比"] = pd.to_numeric(
        big_df["负债-总负债同比"], errors="coerce"
    )
    big_df["资产负债率"] = pd.to_numeric(big_df["资产负债率"], errors="coerce")
    big_df["股东权益合计"] = pd.to_numeric(big_df["股东权益合计"], errors="coerce")
    big_df["股东权益合计"] = pd.to_numeric(big_df["股东权益合计"], errors="coerce")
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    return big_df


def stock_zcfz_bj_em(date: str = "20240331") -> pd.DataFrame:
    """
    东方财富-数据中心-年报季报-业绩快报-资产负债表
    https://data.eastmoney.com/bbsj/202003/zcfz.html
    :param date: choice of {"20200331", "20200630", "20200930", "20201231", "..."}; 从 20100331 开始
    :type date: str
    :return: 资产负债表
    :rtype: pandas.DataFrame
    """
    import warnings

    warnings.filterwarnings(action="ignore", category=FutureWarning)
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_DMSK_FN_BALANCE",
        "columns": "ALL",
        "filter": f"""(TRADE_MARKET_CODE="069001017")
        (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], join="outer", ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "_",
        "股票代码",
        "_",
        "_",
        "股票简称",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "公告日期",
        "_",
        "资产-总资产",
        "_",
        "资产-货币资金",
        "_",
        "资产-应收账款",
        "_",
        "资产-存货",
        "_",
        "负债-总负债",
        "负债-应付账款",
        "_",
        "负债-预收账款",
        "_",
        "股东权益合计",
        "_",
        "资产-总资产同比",
        "负债-总负债同比",
        "_",
        "资产负债率",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "资产-货币资金",
            "资产-应收账款",
            "资产-存货",
            "资产-总资产",
            "资产-总资产同比",
            "负债-应付账款",
            "负债-预收账款",
            "负债-总负债",
            "负债-总负债同比",
            "资产负债率",
            "股东权益合计",
            "公告日期",
        ]
    ]

    big_df["资产-货币资金"] = pd.to_numeric(big_df["资产-货币资金"], errors="coerce")
    big_df["资产-应收账款"] = pd.to_numeric(big_df["资产-应收账款"], errors="coerce")
    big_df["资产-存货"] = pd.to_numeric(big_df["资产-存货"], errors="coerce")
    big_df["资产-总资产"] = pd.to_numeric(big_df["资产-总资产"], errors="coerce")
    big_df["资产-总资产同比"] = pd.to_numeric(
        big_df["资产-总资产同比"], errors="coerce"
    )
    big_df["负债-应付账款"] = pd.to_numeric(big_df["负债-应付账款"], errors="coerce")
    big_df["负债-预收账款"] = pd.to_numeric(big_df["负债-预收账款"], errors="coerce")
    big_df["负债-总负债"] = pd.to_numeric(big_df["负债-总负债"], errors="coerce")
    big_df["负债-总负债同比"] = pd.to_numeric(
        big_df["负债-总负债同比"], errors="coerce"
    )
    big_df["资产负债率"] = pd.to_numeric(big_df["资产负债率"], errors="coerce")
    big_df["股东权益合计"] = pd.to_numeric(big_df["股东权益合计"], errors="coerce")
    big_df["股东权益合计"] = pd.to_numeric(big_df["股东权益合计"], errors="coerce")
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    return big_df


def stock_lrb_em(date: str = "20240331") -> pd.DataFrame:
    """
    东方财富-数据中心-年报季报-业绩快报-利润表
    https://data.eastmoney.com/bbsj/202003/lrb.html
    :param date: choice of {"20200331", "20200630", "20200930", "20201231", "..."}; 从 20100331 开始
    :type date: str
    :return: 利润表
    :rtype: pandas.DataFrame
    """
    import warnings

    warnings.filterwarnings(action="ignore", category=FutureWarning)
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_DMSK_FN_INCOME",
        "columns": "ALL",
        "filter": f"""(SECURITY_TYPE_CODE in ("058001001","058001008"))(TRADE_MARKET_CODE!="069001017")
        (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "_",
        "股票代码",
        "_",
        "_",
        "股票简称",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "公告日期",
        "_",
        "净利润",
        "营业总收入",
        "营业总支出-营业总支出",
        "_",
        "营业总支出-营业支出",
        "_",
        "_",
        "营业总支出-销售费用",
        "营业总支出-管理费用",
        "营业总支出-财务费用",
        "营业利润",
        "利润总额",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "营业总收入同比",
        "_",
        "净利润同比",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "净利润",
            "净利润同比",
            "营业总收入",
            "营业总收入同比",
            "营业总支出-营业支出",
            "营业总支出-销售费用",
            "营业总支出-管理费用",
            "营业总支出-财务费用",
            "营业总支出-营业总支出",
            "营业利润",
            "利润总额",
            "公告日期",
        ]
    ]

    big_df["净利润"] = pd.to_numeric(big_df["净利润"], errors="coerce")
    big_df["净利润同比"] = pd.to_numeric(big_df["净利润同比"], errors="coerce")
    big_df["营业总收入"] = pd.to_numeric(big_df["营业总收入"], errors="coerce")
    big_df["营业总收入同比"] = pd.to_numeric(big_df["营业总收入同比"], errors="coerce")
    big_df["营业总支出-营业支出"] = pd.to_numeric(
        big_df["营业总支出-营业支出"], errors="coerce"
    )
    big_df["营业总支出-销售费用"] = pd.to_numeric(
        big_df["营业总支出-销售费用"], errors="coerce"
    )
    big_df["营业总支出-管理费用"] = pd.to_numeric(
        big_df["营业总支出-管理费用"], errors="coerce"
    )
    big_df["营业总支出-财务费用"] = pd.to_numeric(
        big_df["营业总支出-财务费用"], errors="coerce"
    )
    big_df["营业总支出-营业总支出"] = pd.to_numeric(
        big_df["营业总支出-营业总支出"], errors="coerce"
    )
    big_df["营业利润"] = pd.to_numeric(big_df["营业利润"], errors="coerce")
    big_df["利润总额"] = pd.to_numeric(big_df["利润总额"], errors="coerce")
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date

    return big_df


def stock_xjll_em(date: str = "20240331") -> pd.DataFrame:
    """
    东方财富-数据中心-年报季报-业绩快报-现金流量表
    https://data.eastmoney.com/bbsj/202003/xjll.html
    :param date: choice of {"20200331", "20200630", "20200930", "20201231", "..."}; 从 20100331 开始
    :type date: str
    :return: 现金流量表
    :rtype: pandas.DataFrame
    """
    import warnings

    warnings.filterwarnings(action="ignore", category=FutureWarning)
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_DMSK_FN_CASHFLOW",
        "columns": "ALL",
        "filter": f"""(SECURITY_TYPE_CODE in ("058001001","058001008"))(TRADE_MARKET_CODE!="069001017")
        (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "_",
        "股票代码",
        "_",
        "_",
        "股票简称",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "公告日期",
        "_",
        "经营性现金流-现金流量净额",
        "经营性现金流-净现金流占比",
        "_",
        "_",
        "_",
        "_",
        "投资性现金流-现金流量净额",
        "投资性现金流-净现金流占比",
        "_",
        "_",
        "_",
        "_",
        "融资性现金流-现金流量净额",
        "融资性现金流-净现金流占比",
        "净现金流-净现金流",
        "净现金流-同比增长",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "净现金流-净现金流",
            "净现金流-同比增长",
            "经营性现金流-现金流量净额",
            "经营性现金流-净现金流占比",
            "投资性现金流-现金流量净额",
            "投资性现金流-净现金流占比",
            "融资性现金流-现金流量净额",
            "融资性现金流-净现金流占比",
            "公告日期",
        ]
    ]

    big_df["净现金流-净现金流"] = pd.to_numeric(
        big_df["净现金流-净现金流"], errors="coerce"
    )
    big_df["净现金流-同比增长"] = pd.to_numeric(
        big_df["净现金流-同比增长"], errors="coerce"
    )
    big_df["经营性现金流-现金流量净额"] = pd.to_numeric(
        big_df["经营性现金流-现金流量净额"], errors="coerce"
    )
    big_df["经营性现金流-净现金流占比"] = pd.to_numeric(
        big_df["经营性现金流-净现金流占比"], errors="coerce"
    )
    big_df["投资性现金流-现金流量净额"] = pd.to_numeric(
        big_df["投资性现金流-现金流量净额"], errors="coerce"
    )
    big_df["投资性现金流-净现金流占比"] = pd.to_numeric(
        big_df["投资性现金流-净现金流占比"], errors="coerce"
    )
    big_df["融资性现金流-现金流量净额"] = pd.to_numeric(
        big_df["融资性现金流-现金流量净额"], errors="coerce"
    )
    big_df["融资性现金流-净现金流占比"] = pd.to_numeric(
        big_df["融资性现金流-净现金流占比"], errors="coerce"
    )
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    return big_df


if __name__ == "__main__":
    stock_zcfz_em_df = stock_zcfz_em(date="20240331")
    print(stock_zcfz_em_df)

    stock_zcfz_bj_em_df = stock_zcfz_bj_em(date="20240331")
    print(stock_zcfz_bj_em_df)

    stock_lrb_em_df = stock_lrb_em(date="20240331")
    print(stock_lrb_em_df)

    stock_xjll_em_df = stock_xjll_em(date="20240331")
    print(stock_xjll_em_df)
